package cn.superdesk.data.extract.entity.opCenter;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import lombok.Builder;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;

/**
 * <p>
 * 运营中心-订单商品表
 * </p>
 *
 * @author storyxc
 * @since 2023-03-17
 */
@Data
@EqualsAndHashCode(callSuper = false)
@TableName("tb_operation_center_order_goods")
@Builder(toBuilder = true)
public class OperationCenterOrderGoods implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 自增id
     */
    @TableId(value = "id", type = IdType.AUTO)
    private Long id;

    /**
     * 业务类型：1-订单，2-售后 取商品所属订单的业务类型
     */
    @TableField("biz_type_id")
    private Integer bizTypeId;

    /**
     * 订单商品id：取商品所属订单的订单商品order_goods_id,同订单内唯一
     */
    @TableField("order_goods_id")
    private String orderGoodsId;

    /**
     * 订单编号：取业务方订单商品所属所属订单/售后的订单编号
     */
    @TableField("order_sn")
    private String orderSn;

    /**
     * 订单id：取业务方订单商品所属所属订单/售后的订单id
     */
    @TableField("order_id")
    private String orderId;

    /**
     * 父订单编号：取订单商品所属订单的父订单编号
     */
    @TableField("parent_order_sn")
    private String parentOrderSn;

    /**
     * 父订单id：取订单商品所属订单的业务方父订单id
     */
    @TableField("parent_order_id")
    private String parentOrderId;

    /**
     * 第三方订单号：取订单/售后时第三方订单编号内容：京东子单号、超级前台供应链订单编号、蜂助手订单编号、海威、友宝等
     */
    @TableField("third_party_order_sn")
    private String thirdPartyOrderSn;

    /**
     * 业务平台编码 ①FLSC：福利商城②FWZC：服务之窗
     */
    @TableField("biz_platform_code")
    private String bizPlatformCode;

    /**
     * 业务平台名称 ①FLSC：福利商城②FWZC：服务之窗
     */
    @TableField("biz_platform_name")
    private String bizPlatformName;

    /**
     * 订单类型编码1）FLSC：福利商城（2）FZS：蜂助手
     */
    @TableField("order_type_code")
    private String orderTypeCode;

    /**
     * 订单类型名称1）FLSC：福利商城（2）FZS：蜂助手
     */
    @TableField("order_type_name")
    private String orderTypeName;

    /**
     * 订单时间，订单-下单时间 售后单-售后创建时间
     */
    @TableField("order_create_time")
    private Date orderCreateTime;

    /**
     * 订单完成时间 订单/售后在业务方订单最终状态（已完成/已驳回/已取消/已作废等）的完成时间，若该订单未完成前为空；
     */
    @TableField("finish_time")
    private Date finishTime;

    /**
     * 是否套餐 业务方定义：0-否 1-是 若业务方无此内容，均默认为“0-否”；
     */
    @TableField("is_package")
    private Integer isPackage;

    /**
     * 套餐id 业务方该订单商品所属的套餐id
     */
    @TableField("package_id")
    private String packageId;

    /**
     * 套餐名称 业务方该订单商品所属的套餐名称内容
     */
    @TableField("package_name")
    private String packageName;

    /**
     * 商品类型id 运营中心定义：1-商品，2-运费，3-商户承担运费，4-赠品
     * （1）商品：指该订单商品数据为商品；
     * （2）运费：指该订单商品数据为用户承担的运费；
     * （3）商户承担运费：指该订单商品数据为商户承担的运费；
     * （4）赠品：指该订单商品数据为赠品，目前业务方暂无此类数据，预留；
     */
    @TableField("goods_type_id")
    private Integer goodsTypeId;

    /**
     * 商品id：业务方该订单商品id
     */
    @TableField("goods_id")
    private String goodsId;

    /**
     * 商品sku：业务方该订单商品sku编码
     */
    @TableField("goods_sku")
    private String goodsSku;

    /**
     * 商品名称：（1）福利商城：
     * ①商品/赠品：业务方该订单商品名称内容；
     * ②运费：统一默认为“运费”；
     * ③商户承担运费：统一默认为“商户承担运费”；
     * （2）卡劵商城：统一默认为“卡劵类商品”；
     */
    @TableField("goods_name")
    private String goodsName;

    /**
     * 商品数量：业务方订单/售后的商品数量
     * （1）订单：该订单商品数量为正数，允许为0；
     * （2）售后（退货退款/退款）：该该订单商品退货退款的数量为负数，允许为0，此值若业务方为正数，需转换为负数；
     * （3）售后（换货/维修）：该该订单商品（换货/维修）数量保持原业务换货/维修的数量值不变；
     */
    @TableField("quantity")
    private Integer quantity;

    /**
     * 商品单位：该订单商品的商品单位，若单位为空时，入库到运营中心默认为“项”。
     */
    @TableField("unit")
    private String unit;

    /**
     * 商品原单价：该订单商品的原销售单价（折扣前）
     * （1）订单：业务方该订单商品的原销售单价（折扣前），该金额≥0.00；
     * （2）售后（退货退款）：业务方该订单商品（退货退款）的原销售单价（折扣前），此值若业务方为正数，需转换为负数；
     * （3）售后（换货/维修）：默认为0.00；
     */
    @TableField("original_unit_price")
    private BigDecimal originalUnitPrice;

    /**
     * 商品原总价：该订单商品的原销售总价（折扣前）
     * （1）订单：业务方该订单商品的原销售总价（折扣前），该金额≥0.00；
     * （2）售后（退货退款）：业务方该订单商品（退货退款）的原销售总价（折扣前），此值若业务方为正数，需转换为负数；
     * （3）售后（换货/维修）：默认为0.00；
     */
    @TableField("original_total_price")
    private BigDecimal originalTotalPrice;

    /**
     * 商品销售单价：（1）订单、售后（退货退款）：该订单商品的销售单价（折扣后）
     * （2）售后（换货/维修）：默认为0.00；
     */
    @TableField("selling_unit_price")
    private BigDecimal sellingUnitPrice;

    /**
     * 商品销售总价：该订单商品的销售总价（折扣后）
     * （1）订单：业务方该订单商品的销售总价（折扣后），该金额≥0.00；
     * （2）售后（退货退款）：业务方该订单商品（退货退款）的销售总价（折扣后），此值若业务方为正数，需转换为负数；
     * （3）售后（换货/维修）：默认为0.00；
     */
    @TableField("selling_total_price")
    private BigDecimal sellingTotalPrice;

    /**
     * 商品协议单价：（1）订单、售后（退货退款）：该订单商品的协议单价；
     * （2）售后（换货/维修）：默认为0.00；
     */
    @TableField("negotiated_unit_price")
    private BigDecimal negotiatedUnitPrice;

    /**
     * 商品协议总价：该订单商品的协议总价
     * （1）订单：业务方该订单商品的协议总价，该金额≥0.00；
     * （2）售后（退货退款）：业务方该订单商品（退货退款）的协议总价，此值若业务方为正数，需转换为负数；
     * （3）售后（换货/维修）：默认为0.00；
     */
    @TableField("negotiated_total_price")
    private BigDecimal negotiatedTotalPrice;

    /**
     * 订单商品金额（实付金额/实际退款金额）：该订单商品的实付/退款金额
     */
    @TableField("order_goods_amount")
    private BigDecimal orderGoodsAmount;

    /**
     * 商品结算单价：基于运营商规则取值
     */
    @TableField("settlement_unit_price")
    private BigDecimal settlementUnitPrice;

    /**
     * 商品结算金额：基于运营商规则取值
     */
    @TableField("settlement_amount")
    private BigDecimal settlementAmount;

    /**
     * 商品成本单价：（1）订单、售后（退货退款）：该订单商品下单时商品成本单价；
     * （2）售后（换货/维修）：默认为0.00；
     */
    @TableField("cost_unit_price")
    private BigDecimal costUnitPrice;

    /**
     * 商品成本总价：该订单商品下单时的供应商成本总价
     * （1）订单：业务方该订单商品的供应商成本总价，该金额≥0.00；
     * （2）售后（退货退款）：业务方该订单商品（退货退款）的供应商成本总价，此值若业务方为正数，需转换为负数；
     * （3）售后（换货/维修）：默认为0.00；
     */
    @TableField("cost_total_price")
    private BigDecimal costTotalPrice;

    /**
     * 商品优惠金额：该订单商品的优惠金额总和
     * （1）订单：业务方该订单商品的所有优惠项优惠金额总和；
     * （2）售后（退货退款）：业务方该订单商品（退货退款）的所有优惠项优惠金额总和，此值若业务方为正数，需转换为负数；
     * （3）售后（换货/维修）：默认为0.00；
     * （4）若金额为空时，需入库时为0.00；
     */
    @TableField("discount_amount")
    private BigDecimal discountAmount;

    /**
     * 平台税务编码
     */
    @TableField("platform_tax_code")
    private String platformTaxCode;

    /**
     * 平台税率
     */
    @TableField("platform_tax_rate")
    private BigDecimal platformTaxRate;

    /**
     * 商户税务编码
     */
    @TableField("merchant_tax_code")
    private String merchantTaxCode;

    /**
     * 商户税率
     */
    @TableField("merchant_tax_rate")
    private BigDecimal merchantTaxRate;

    /**
     * 是否免税商品 0-不免税 1-免税
     */
    @TableField("is_duty_free")
    private Integer isDutyFree;

    /**
     * 运营商id
     */
    @TableField("operator_id")
    private String operatorId;

    /**
     * 运营商名称
     */
    @TableField("operator_name")
    private String operatorName;

    /**
     * 供应商id
     */
    @TableField("supplier_id")
    private String supplierId;

    /**
     * 供应商名称
     */
    @TableField("supplier_name")
    private String supplierName;

    /**
     * 品牌id
     */
    @TableField("brand_id")
    private String brandId;

    /**
     * 品牌名称
     */
    @TableField("brand_name")
    private String brandName;

    /**
     * 一级分类id
     */
    @TableField("cate_level_1_id")
    private String cateLevel1Id;

    /**
     * 一级分类名称
     */
    @TableField("cate_level_1_name")
    private String cateLevel1Name;

    /**
     * 二级分类id
     */
    @TableField("cate_level_2_id")
    private String cateLevel2Id;

    /**
     * 二级分类名称
     */
    @TableField("cate_level_2_name")
    private String cateLevel2Name;

    /**
     * 三级分类id
     */
    @TableField("cate_level_3_id")
    private String cateLevel3Id;

    /**
     * 三级分类名称
     */
    @TableField("cate_level_3_name")
    private String cateLevel3Name;

    /**
     * 商品规格或型号
     */
    @TableField("goods_specification")
    private String goodsSpecification;

    /**
     * 其他属性 支持json存储
     */
    @TableField("other_attribute")
    private String otherAttribute;

    /**
     * 退单原单号
     */
    @TableField("refund_origin_order_sn")
    private String refundOriginOrderSn;

    /**
     * 创建人id  该数据入库到运营中心的人id，初始一般是系统
     */
    @TableField("create_user_id")
    private String createUserId;

    /**
     * 创建人姓名 该数据入库到运营中心的人，初始一般是系统
     */
    @TableField("create_user_name")
    private String createUserName;

    /**
     * 创建时间
     */
    @TableField("create_time")
    private Date createTime;

    /**
     * 更新人id
     */
    @TableField("update_user_id")
    private String updateUserId;

    /**
     * 更新人姓名
     */
    @TableField("update_user_name")
    private String updateUserName;

    /**
     * 更新时间
     */
    @TableField("update_time")
    private Date updateTime;


}
