package cn.superdesk.data.extract.entity.opCenter;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;

/**
 * <p>
 * 运营中心-订单流水表
 * </p>
 *
 * @author storyxc
 * @since 2023-04-18
 */
@Data
@EqualsAndHashCode(callSuper = false)
@TableName("tb_operation_center_order_transaction_log")
public class OperationCenterOrderTransactionLog implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 自增主键
     */
    @TableId(value = "id", type = IdType.AUTO)
    private Long id;

    /**
     * 业务类型id 1-订单，2-售后
     */
    @TableField("biz_type_id")
    private Integer bizTypeId;

    /**
     * 订单商品id
     */
    @TableField("order_goods_id")
    private String orderGoodsId;

    /**
     * 订单编号
     */
    @TableField("order_sn")
    private String orderSn;

    /**
     * 订单id
     */
    @TableField("order_id")
    private String orderId;

    /**
     * 业务流水号
     */
    @TableField("biz_transaction_number")
    private String bizTransactionNumber;

    /**
     * 第三方流水号
     */
    @TableField("third_party_transaction_number")
    private String thirdPartyTransactionNumber;

    /**
     * 业务平台编码 ①FLSC：福利商城 ②FWZC：服务之窗
     */
    @TableField("biz_platform_code")
    private String bizPlatformCode;

    /**
     * 业务平台名称  ①FLSC：福利商城 ②FWZC：服务之窗
     */
    @TableField("biz_platform_name")
    private String bizPlatformName;

    /**
     * 订单类型编码
     */
    @TableField("order_type_code")
    private String orderTypeCode;

    /**
     * 订单类型名称
     */
    @TableField("order_type_name")
    private String orderTypeName;

    /**
     * 商品类型id 运营中心定义：1-商品，2-运费，3-商户承担运费，4-赠品
     */
    @TableField("goods_type_id")
    private Integer goodsTypeId;

    /**
     * 商品id
     */
    @TableField("goods_id")
    private String goodsId;

    /**
     * 商品sku/编码
     */
    @TableField("goods_sku")
    private String goodsSku;

    /**
     * 商品名称
     */
    @TableField("goods_name")
    private String goodsName;

    /**
     * 商品单位
     */
    @TableField("unit")
    private String unit;

    /**
     * 结算金额 （1）订单：取该商品订单流水的支付方式实际用户支付的金额；
     * （2）售后：取该商品订单流水的支付方式实际退款的金额，负数，此值若业务方为正数，需转换为负数；
     */
    @TableField("settlement_amount")
    private BigDecimal settlementAmount;

    /**
     * 积分类型id
     */
    @TableField("integral_type_id")
    private Integer integralTypeId;

    /**
     * 积分类型名称
     */
    @TableField("integral_type_name")
    private String integralTypeName;

    /**
     * （1）订单：取该订单流水支付时的积分批次；
     * （2）售后：取该订单流水退款时的积分批次；
     */
    @TableField("batch")
    private String batch;

    /**
     * 交易人id
     */
    @TableField("transactor_id")
    private String transactorId;

    /**
     * 交易人姓名
     */
    @TableField("transactor_name")
    private String transactorName;

    /**
     * 交易人手机号
     */
    @TableField("transactor_mobile")
    private String transactorMobile;

    /**
     * 企业id
     */
    @TableField("enterprise_id")
    private String enterpriseId;

    /**
     * 企业名称
     */
    @TableField("enterprise_name")
    private String enterpriseName;

    /**
     * 项目id
     */
    @TableField("org_id")
    private String orgId;

    /**
     * 项目名称
     */
    @TableField("org_name")
    private String orgName;

    /**
     * 运营商id
     */
    @TableField("operator_id")
    private String operatorId;

    /**
     * 运营商名称
     */
    @TableField("operator_name")
    private String operatorName;

    /**
     * 业务支付方式id
     */
    @TableField("biz_pay_type_id")
    private String bizPayTypeId;

    /**
     * 业务支付方式名称
     */
    @TableField("biz_pay_type_name")
    private String bizPayTypeName;

    /**
     * 平台支付方式id
     */
    @TableField("platform_pay_type_id")
    private Long platformPayTypeId;

    /**
     * 平台支付方式名称
     */
    @TableField("platform_pay_type_name")
    private String platformPayTypeName;

    /**
     * 流水支付状态id：运营中心定义：1-未支付，2-已支付，3-未退款，4-已退款；
     */
    @TableField("pay_status_id")
    private Integer payStatusId;

    /**
     * 业务支付渠道id
     */
    @TableField("biz_pay_channel_id")
    private String bizPayChannelId;

    /**
     * 业务支付渠道名称
     */
    @TableField("biz_pay_channel_name")
    private String bizPayChannelName;

    /**
     * 支付手续费
     */
    @TableField("transaction_fee")
    private BigDecimal transactionFee;

    /**
     * 订单时间
     */
    @TableField("order_create_time")
    private Date orderCreateTime;

    /**
     * 支付/退款时间
     */
    @TableField("trading_time")
    private Date tradingTime;

    /**
     * 完成时间
     */
    @TableField("finish_time")
    private Date finishTime;

    /**
     * 退单原单号
     */
    @TableField("refund_origin_order_sn")
    private String refundOriginOrderSn;

    /**
     * 创建人id
     */
    @TableField("create_user_id")
    private String createUserId;

    /**
     * 创建人
     */
    @TableField("create_user_name")
    private String createUserName;

    /**
     * 创建时间
     */
    @TableField("create_time")
    private Date createTime;


}
