package cn.superdesk.data.extract.job.flsc;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.thread.ThreadUtil;
import cn.superdesk.data.extract.dto.JobParamDTO;
import cn.superdesk.data.extract.entity.opCenter.OperationDataSyncErrorRecord;
import cn.superdesk.data.extract.entity.flsc.FlscShopOperatorsUser;
import cn.superdesk.data.extract.entity.opCenter.UnifiedOperators;
import cn.superdesk.data.extract.enums.opCenter.OpCenterBizPlatformEnum;
import cn.superdesk.data.extract.enums.opCenter.OperationDataSyncErrorHandleStatusEnum;
import cn.superdesk.data.extract.enums.opCenter.OperationDataSyncTypeEnum;
import cn.superdesk.data.extract.mapper.flsc.FlscShopOperatorsUserMapper;
import cn.superdesk.data.extract.mapper.opCenter.OperationCenterOperatorsMapper;
import cn.superdesk.data.extract.service.opCenter.OperationDataSyncErrorRecordService;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author xc
 * @description 福利商城运营商增量同步任务
 * @date 2023/3/17 18:26
 */
@Component
@Slf4j
@RequiredArgsConstructor
public class FlscOperatorSyncJob {
    private final OperationCenterOperatorsMapper operationCenterOperatorsMapper;
    private final FlscShopOperatorsUserMapper flscShopOperatorsUserMapper;
    private final OperationDataSyncErrorRecordService operationDataSyncErrorRecordService;

    @XxlJob(("flscOperatorSyncHandler"))
    @Transactional(rollbackFor = Exception.class)
    public void flscOperatorSync() {
        String startTime = DateUtil.now();
        String jobParam = XxlJobHelper.getJobParam();
        log.info("任务参数 : {}", jobParam);
        XxlJobHelper.log("任务参数 : {}", jobParam);
        // 解析参数
        JobParamDTO jobParamDTO = null;
        // 运营中心运营商待新增列表
        List<UnifiedOperators> unifiedOperatorsInsertList = new ArrayList<>();
        // 运营中心运营商待修改列表
        List<UnifiedOperators> unifiedOperatorsUpdateList = new ArrayList<>();
        // 根据不同条件查询出的福利商城运营商列表
        List<FlscShopOperatorsUser> flscShopOperatorsUsers = null;
        if (StringUtils.isNotBlank(jobParam)) {
            try {
                jobParamDTO = JSON.parseObject(jobParam, JobParamDTO.class);
            } catch (Exception e) {
                log.info("任务参数解析失败 参数内容: {}", jobParam);
                XxlJobHelper.log("任务参数解析失败 参数内容: {}", jobParam);
                throw new RuntimeException("任务参数解析失败");
            }

            if (jobParamDTO == null || (StringUtils.isBlank(jobParamDTO.getMainParam()) && StringUtils.isBlank(jobParamDTO.getSyncTime()))) {
                log.info("任务参数解析失败 参数内容: {}", jobParam);
                XxlJobHelper.log("任务参数解析失败 参数内容: {}", jobParam);
                throw new RuntimeException("任务参数解析失败");
            }
            String syncTime = jobParamDTO.getSyncTime();
            String mainParam = jobParamDTO.getMainParam();
            if (StringUtils.isNotBlank(syncTime)) {
                // 按照指定时间同步
                Date parse;
                try {
                    parse = DateUtil.parse(syncTime, "yyyy-MM-dd HH:mm:ss");
                } catch (Exception e) {
                    log.info("日期参数解析失败 任务失败 参数内容: {}", jobParam);
                    XxlJobHelper.log("日期参数解析失败 任务失败 参数内容: {}", jobParam);
                    throw new RuntimeException("日期参数解析失败");
                }
                log.info("按照指定时间同步 参数 : {}", syncTime);

                flscShopOperatorsUsers = flscShopOperatorsUserMapper.selectList(
                        Wrappers.<FlscShopOperatorsUser>lambdaQuery()
                                .ge(FlscShopOperatorsUser::getSyncTime, parse)
                );
            } else {
                // 按照唯一标识同步
                log.info("按照唯一标识同步 参数 : {}", mainParam);
                flscShopOperatorsUsers = flscShopOperatorsUserMapper.selectList(
                        Wrappers.<FlscShopOperatorsUser>lambdaQuery()
                                .eq(FlscShopOperatorsUser::getId, mainParam)
                );
            }

        } else {
            // 获取五分钟前的Date信息
            Date fiveMinutesAgo = DateUtil.offsetMinute(new Date(), -5);
            // 获取五分钟内变化的数据，实际定时任务每分钟执行一次
            flscShopOperatorsUsers = flscShopOperatorsUserMapper.selectList(
                    Wrappers.<FlscShopOperatorsUser>lambdaQuery()
                            .ge(FlscShopOperatorsUser::getSyncTime, fiveMinutesAgo)
            );
        }
        if (CollectionUtil.isNotEmpty(flscShopOperatorsUsers)) {
            for (FlscShopOperatorsUser flscShopOperatorsUser : flscShopOperatorsUsers) {

                UnifiedOperators syncOperationCenterOperator = UnifiedOperators.builder()
                        .name(flscShopOperatorsUser.getOperatorsName())// 运营商名称
                        .bizPlatformCode(OpCenterBizPlatformEnum.福利商城.getCode())// 业务平台编码
                        .bizPlatformName(OpCenterBizPlatformEnum.福利商城.getName())// 业务平台名称
                        .bizOperatorId(String.valueOf(flscShopOperatorsUser.getId()))// 业务方运营商id
                        .bizOperatorCode(flscShopOperatorsUser.getOperatorsNo())// 运营商编号
                        .contact(flscShopOperatorsUser.getRealname())// 联系人
                        .mobile(flscShopOperatorsUser.getMobile())// 联系电话
                        .address(flscShopOperatorsUser.getAddress())// 地址
                        .remark(flscShopOperatorsUser.getRemark())// 备注
                        .latitude(flscShopOperatorsUser.getLat())// 经度
                        .longitude(flscShopOperatorsUser.getLng())// 纬度
                        .deleted(0)// 是否删除 福利商城默认0
                        .build();

                // 查询运营中心是否存在该运营商
                UnifiedOperators unifiedOperators = operationCenterOperatorsMapper.selectOne(
                        Wrappers.<UnifiedOperators>lambdaQuery()
                                .eq(UnifiedOperators::getBizOperatorId, flscShopOperatorsUser.getId())
                                .eq(UnifiedOperators::getBizPlatformCode, OpCenterBizPlatformEnum.福利商城.getCode())
                );
                if (unifiedOperators != null && unifiedOperators.getId() != null) {
                    // 修改
                    syncOperationCenterOperator = syncOperationCenterOperator.toBuilder()
                            .id(unifiedOperators.getId())
                            .updateTime(new Date())
                            .updateUserId(0L)
                            .build();
                    unifiedOperatorsUpdateList.add(syncOperationCenterOperator);

                } else {
                    // 新增
                    syncOperationCenterOperator = syncOperationCenterOperator.toBuilder()
                            .createTime(new Date())
                            .createUserId(0L)
                            .build();
                    unifiedOperatorsInsertList.add(syncOperationCenterOperator);
                }
            }
        }

        int errorCount = 0;

        if (CollectionUtil.isNotEmpty(unifiedOperatorsInsertList)) {
            for (UnifiedOperators unifiedOperators : unifiedOperatorsInsertList) {
                try {
                    operationCenterOperatorsMapper.insert(unifiedOperators);
                } catch (Exception e) {
                    // 异常数据记录在表
                    ThreadUtil.execute(() -> {
                        operationDataSyncErrorRecordService.save(
                                OperationDataSyncErrorRecord.builder()
                                        .syncType(OperationDataSyncTypeEnum.运营商.getName())
                                        .bizPlatformCode(OpCenterBizPlatformEnum.福利商城.getCode())
                                        .bizPlatformName(OpCenterBizPlatformEnum.福利商城.getName())
                                        .mainParam(unifiedOperators.getBizOperatorId())
                                        .syncTime(new Date())
                                        .errMsg(JSON.toJSONString(e))
                                        .retentionData(JSON.toJSONString(unifiedOperators))
                                        .handleStatus(OperationDataSyncErrorHandleStatusEnum.未处理.getCode())
                                        .build()
                        );
                    });
                    log.info("运营商新增失败，运营商信息: {}, 参数信息: {}, 错误信息: {}", JSON.toJSONString(unifiedOperators), jobParam, e);
                    errorCount += 1;
                }
            }
            // operationCenterOperatorsService.saveBatch(operationCenterOperatorsInsertList);
        }
        if (CollectionUtil.isNotEmpty(unifiedOperatorsUpdateList)) {
            for (UnifiedOperators unifiedOperators : unifiedOperatorsUpdateList) {
                try {
                    operationCenterOperatorsMapper.updateById(unifiedOperators);
                } catch (Exception e) {
                    // 异常数据需要记录在表
                    ThreadUtil.execute(() -> {
                        operationDataSyncErrorRecordService.save(
                                OperationDataSyncErrorRecord.builder()
                                        .syncType(OperationDataSyncTypeEnum.运营商.getName())
                                        .bizPlatformCode(OpCenterBizPlatformEnum.福利商城.getCode())
                                        .bizPlatformName(OpCenterBizPlatformEnum.福利商城.getName())
                                        .mainParam(unifiedOperators.getBizOperatorId())
                                        .syncTime(new Date())
                                        .errMsg(JSON.toJSONString(e))
                                        .retentionData(JSON.toJSONString(unifiedOperators))
                                        .handleStatus(OperationDataSyncErrorHandleStatusEnum.未处理.getCode())
                                        .build()
                        );
                    });
                    log.info("运营商修改失败，运营商信息: {}, 参数信息: {}, 错误信息: {}", JSON.toJSONString(unifiedOperators), jobParam, e);
                    errorCount += 1;
                }
            }
        }

        log.info("福利商城运营商增量同步任务执行成功，本次同步运营商数： {}， 总失败记录数: {}, 开始执行时间: {} , 结束时间: {}, 参数内容: {}", flscShopOperatorsUsers.size(), errorCount, startTime, DateUtil.now(), jobParam);


    }
}
