package cn.superdesk.data.extract.job.flsc;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.thread.ThreadUtil;
import cn.superdesk.data.extract.dto.JobParamDTO;
import cn.superdesk.data.extract.entity.opCenter.OperationDataSyncErrorRecord;
import cn.superdesk.data.extract.entity.flsc.FlscShopOrder;
import cn.superdesk.data.extract.entity.flsc.FlscShopOrderRefund;
import cn.superdesk.data.extract.entity.opCenter.OperationCenterOrder;
import cn.superdesk.data.extract.entity.opCenter.OperationCenterOrderGoods;
import cn.superdesk.data.extract.enums.OpCenterJobSyncTypeEnum;
import cn.superdesk.data.extract.enums.flsc.FlscAfterSaleStatusEnum;
import cn.superdesk.data.extract.enums.opCenter.*;
import cn.superdesk.data.extract.mapper.flsc.FlscShopOrderMapper;
import cn.superdesk.data.extract.mapper.flsc.FlscShopOrderRefundMapper;
import cn.superdesk.data.extract.mapper.opCenter.OperationCenterOrderGoodsMapper;
import cn.superdesk.data.extract.mapper.opCenter.OperationCenterOrderMapper;
import cn.superdesk.data.extract.service.flsc.FlscShopOrderRefundService;
import cn.superdesk.data.extract.service.opCenter.OperationDataSyncErrorRecordService;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author xc
 * @description
 * @date 2023/3/27 16:02
 */
@Component
@Slf4j
@RequiredArgsConstructor
public class FlscRefundOrderJob {
    private final FlscShopOrderRefundMapper flscShopOrderRefundMapper;
    private final FlscShopOrderRefundService flscShopOrderRefundService;
    private final FlscShopOrderMapper flscShopOrderMapper;
    private final OperationCenterOrderMapper operationCenterOrderMapper;
    private final OperationCenterOrderGoodsMapper operationCenterOrderGoodsMapper;
    private final OperationDataSyncErrorRecordService operationDataSyncErrorRecordService;

    /**
     * 福利商城退单数据同步 & 退单商品构造
     */
    @XxlJob("flscRefundOrderSyncHandler")
    public void flscRefundOrderSync() {
        String startTime = DateUtil.now();
        String jobParam = XxlJobHelper.getJobParam();
        log.info("任务参数 : {}", jobParam);
        XxlJobHelper.log("任务参数 : {}", jobParam);
        // 解析参数
        JobParamDTO jobParamDTO = null;

        // 福利商城退单列表
        List<FlscShopOrderRefund> flscShopOrderRefundList = null;
        // 变动数据总数
        Integer total = 0;
        // 同步类型
        int syncType;
        // 数据查询参数
        Object syncParam;
        if (StringUtils.isNotBlank(jobParam)) {
            try {
                jobParamDTO = JSON.parseObject(jobParam, JobParamDTO.class);
            } catch (Exception e) {
                log.info("任务参数解析异常", e);
                XxlJobHelper.log("任务参数解析异常", e);
                throw new RuntimeException("任务参数解析异常");
            }
            if (jobParamDTO == null || (StringUtils.isBlank(jobParamDTO.getMainParam()) && StringUtils.isBlank(jobParamDTO.getSyncTime()))) {
                log.info("任务参数解析失败 参数内容: {}", jobParam);
                XxlJobHelper.log("任务参数解析失败 参数内容: {}", jobParam);
                throw new RuntimeException("任务参数解析失败");
            }

            String syncTime = jobParamDTO.getSyncTime();
            String mainParam = jobParamDTO.getMainParam();
            if (StringUtils.isNotBlank(syncTime)) {
                // 按照指定时间同步
                try {
                    syncParam = DateUtil.parse(syncTime);
                } catch (Exception e) {
                    log.info("任务参数解析异常, 参数内容: {}", jobParam);
                    XxlJobHelper.log("任务参数解析异常, 参数内容 : {}", jobParam);
                    throw new RuntimeException("任务参数解析异常");
                }
                // 数据变化量
                total = flscShopOrderRefundMapper.selectCount(
                        Wrappers.<FlscShopOrderRefund>lambdaQuery()
                                .ge(FlscShopOrderRefund::getSyncTime, syncParam)
                );
                syncType = OpCenterJobSyncTypeEnum.按照指定时间同步.getCode();

            } else {
                // 数据变化量
                total = flscShopOrderRefundMapper.selectCount(
                        Wrappers.<FlscShopOrderRefund>lambdaQuery()
                                .eq(FlscShopOrderRefund::getRefundno, mainParam)
                );
                syncParam = mainParam;
                syncType = OpCenterJobSyncTypeEnum.按照唯一标识同步.getCode();


            }
        } else {
            // 获取十五分钟前的时间
            Date fifteenMinutesAgo = DateUtil.offsetMinute(DateUtil.date(), -15);
            // 数据变化量
            total = flscShopOrderRefundMapper.selectCount(
                    Wrappers.<FlscShopOrderRefund>lambdaQuery()
                            .ge(FlscShopOrderRefund::getSyncTime, fifteenMinutesAgo)
            );
            syncParam = fifteenMinutesAgo;
            syncType = OpCenterJobSyncTypeEnum.默认同步.getCode();
        }
        int errorOrderCount = 0;
        int errorOrderGoodsCount = 0;
        int errorOrderFreightCount = 0;
        int batchSize = 1000;

        if (total == 0) {
            // 没有数据变动
            log.info("福利商城退单数据同步 & 退单商品构造 无数据变化");
            XxlJobHelper.log("福利商城退单数据同步 & 退单商品构造 无数据变化");
            return;
        } else if (total <= batchSize) {
            // 数据变动量小于1000
            flscShopOrderRefundList = getRefundOrderListBySyncType(syncType, syncParam);
            batchHandle(flscShopOrderRefundList, errorOrderCount, errorOrderGoodsCount, errorOrderFreightCount);
        } else {
            // 数据变动量大于1000 分批次处理
            int batch = total / batchSize;
            int remainder = total % batchSize;
            int times = remainder == 0 ? batch : batch + 1;
            int offset = 0;
            for (int i = 0; i < times; i++) {
                flscShopOrderRefundList = getRefundOrderListBySyncTypeAndForBatch(syncType, syncParam, offset, batchSize);
                batchHandle(flscShopOrderRefundList, errorOrderCount, errorOrderGoodsCount, errorOrderFreightCount);
                flscShopOrderRefundList = null;
                offset += batchSize;
            }
        }
        log.info("福利商城退单数据同步 & 退单商品构造 完成, 开始时间: {}, 结束时间: {}, 同步类型: {}, 同步参数: {}, 同步数据量: {}, 同步失败数据量: {}, 同步失败商品数据量: {}, 同步失败运费数据量: {}",
                startTime, DateUtil.now(), OpCenterJobSyncTypeEnum.getDescByCode(syncType), jobParam, total, errorOrderCount, errorOrderGoodsCount, errorOrderFreightCount);
    }


    /**
     * 批量处理
     *
     * @param flscShopOrderRefundList
     * @param errorOrderCount
     * @param errorOrderGoodsCount
     * @param errorOrderFreightCount
     */
    private void batchHandle(List<FlscShopOrderRefund> flscShopOrderRefundList, int errorOrderCount, int errorOrderGoodsCount, int errorOrderFreightCount) {
        // 福利商城退单待新增列表
        List<OperationCenterOrder> operationCenterRefundOrderInsertList = new ArrayList<>();
        // 福利商城退单待修改列表
        List<OperationCenterOrder> operationCenterRefundOrderUpdateList = new ArrayList<>();
        // 福利商城退单商品待新增列表
        List<OperationCenterOrderGoods> operationCenterRefundOrderGoodsInsertList = new ArrayList<>();
        // 福利商城退单商品待修改列表
        List<OperationCenterOrderGoods> operationCenterRefundOrderGoodsUpdateList = new ArrayList<>();
        // 福利商城退单退运费商品待新增列表
        List<OperationCenterOrderGoods> operationCenterRefundOrderFreightGoodsInsertList = new ArrayList<>();
        // 福利商城退单退运费商品待修改列表
        List<OperationCenterOrderGoods> operationCenterRefundOrderFreightGoodsUpdateList = new ArrayList<>();

        if (CollectionUtil.isNotEmpty(flscShopOrderRefundList)) {
            for (FlscShopOrderRefund flscShopOrderRefund : flscShopOrderRefundList) {
                /**
                 * 对应正单的主键id 如果小于0 是商城手动维护的退单，不需要处理
                 */
                Integer orderId = flscShopOrderRefund.getOrderid();
                if (orderId < 0) {
                    // 异常记录记录在表
                    ThreadUtil.execute(() -> {
                        operationDataSyncErrorRecordService.save(
                                OperationDataSyncErrorRecord.builder()
                                        .syncType(OperationDataSyncTypeEnum.订单.getName())
                                        .bizPlatformCode(OpCenterBizPlatformEnum.福利商城.getCode())
                                        .bizPlatformName(OpCenterBizPlatformEnum.福利商城.getName())
                                        .mainParam(String.valueOf(flscShopOrderRefund.getId()))
                                        .orderSn(flscShopOrderRefund.getRefundno())
                                        .syncTime(new Date())
                                        .errMsg("退单对应的订单id为负数")
                                        .retentionData(JSON.toJSONString(flscShopOrderRefund))
                                        .handleStatus(OperationDataSyncErrorHandleStatusEnum.未处理.getCode())
                                        .build()
                        );
                    });
                    log.info("退单商品构造失败, 退单号: {}", flscShopOrderRefund.getRefundno());
                    continue;
                }
                FlscShopOrder flscShopOrder = flscShopOrderMapper.selectById(flscShopOrderRefund.getOrderid());
                // 构造福利商城退单
                OperationCenterOrder operationCenterRefundOrder = flscShopOrderRefundService.buildOpCenterOrderFromFlscRefundOrder(flscShopOrderRefund, flscShopOrder);
                // 构造福利商城订单退单商品
                OperationCenterOrderGoods operationCenterRefundOrderGoods = flscShopOrderRefundService.buildOperationCenterRefundOrderGoods(flscShopOrderRefund, operationCenterRefundOrder, flscShopOrder);
                // 构造福利商城退运费商品
                OperationCenterOrderGoods operationCenterRefundOrderGoodsDispatch = null;
                if (BigDecimal.ZERO.compareTo(flscShopOrderRefund.getRefundDispatchprice()) < 0) {
                    operationCenterRefundOrderGoodsDispatch = flscShopOrderRefundService.buildOperationCenterRefundOrderGoodsDispatch(flscShopOrderRefund, operationCenterRefundOrder, flscShopOrder);
                }

                if (operationCenterRefundOrderGoods == null) {
                    // 异常记录记录在表
                    OperationCenterOrder finalOperationCenterRefundOrder = operationCenterRefundOrder;
                    ThreadUtil.execute(() -> {
                        operationDataSyncErrorRecordService.save(
                                OperationDataSyncErrorRecord.builder()
                                        .syncType(OperationDataSyncTypeEnum.订单.getName())
                                        .bizPlatformCode(OpCenterBizPlatformEnum.福利商城.getCode())
                                        .bizPlatformName(OpCenterBizPlatformEnum.福利商城.getName())
                                        .mainParam(String.valueOf(flscShopOrderRefund.getId()))
                                        .orderSn(flscShopOrderRefund.getRefundno())
                                        .syncTime(new Date())
                                        .errMsg("退单对应的订单商品数据不存在")
                                        .retentionData(JSON.toJSONString(finalOperationCenterRefundOrder))
                                        .handleStatus(OperationDataSyncErrorHandleStatusEnum.未处理.getCode())
                                        .build()
                        );
                    });
                    log.info("退单商品构造失败, 退单号: {}", flscShopOrderRefund.getRefundno());
                    continue;
                }
                // 判断福利商城退单是否存在
                OperationCenterOrder operationCenterRefundOrderDB = operationCenterOrderMapper.selectOne(
                        Wrappers.<OperationCenterOrder>lambdaQuery()
                                .eq(OperationCenterOrder::getOrderSn, flscShopOrderRefund.getRefundno())
                                .eq(OperationCenterOrder::getOrderBizTypeId, OpCenterOrderBizTypeEnum.售后.getCode())
                                .eq(OperationCenterOrder::getOrderTypeCode, OpCenterOrderTypeEnum.福利商城.getCode())
                                .eq(OperationCenterOrder::getBizPlatformCode, OpCenterBizPlatformEnum.福利商城.getCode())
                );
                if (operationCenterRefundOrderDB == null || operationCenterRefundOrderDB.getId() == null) {
                    // 福利商城退单不存在
                    operationCenterRefundOrder = operationCenterRefundOrder.toBuilder()
                            .createUserId("0")
                            .createUserName("增量定时任务同步")
                            .createTime(new Date())
                            .build();

                    operationCenterRefundOrderInsertList.add(operationCenterRefundOrder);
                } else {
                    // 福利商城退单存在
                    operationCenterRefundOrder = operationCenterRefundOrder.toBuilder()
                            .id(operationCenterRefundOrderDB.getId())
                            .updateUserId("0")
                            .updateTime(new Date())
                            .updateUserName("增量定时任务同步")
                            .build();
                    operationCenterRefundOrderUpdateList.add(operationCenterRefundOrder);
                }

                // 判断福利商城退单商品是否存在
                OperationCenterOrderGoods operationCenterRefundOrderGoodsDB = operationCenterOrderGoodsMapper.selectOne(
                        Wrappers.<OperationCenterOrderGoods>lambdaQuery()
                                .eq(OperationCenterOrderGoods::getOrderSn, flscShopOrderRefund.getRefundno())
                                .eq(OperationCenterOrderGoods::getBizTypeId, OpCenterOrderBizTypeEnum.售后.getCode())
                                .eq(OperationCenterOrderGoods::getOrderTypeCode, OpCenterOrderTypeEnum.福利商城.getCode())
                                .eq(OperationCenterOrderGoods::getBizPlatformCode, OpCenterBizPlatformEnum.福利商城.getCode())
                                .eq(OperationCenterOrderGoods::getOrderGoodsId, flscShopOrderRefund.getOrderGoodsId())
                );
                if (operationCenterRefundOrderGoodsDB == null || operationCenterRefundOrderGoodsDB.getId() == null) {
                    // 福利商城退单商品不存在
                    operationCenterRefundOrderGoods = operationCenterRefundOrderGoods.toBuilder()
                            .createUserId("0")
                            .createUserName("增量定时任务同步")
                            .createTime(new Date())
                            .build();
                    operationCenterRefundOrderGoodsInsertList.add(operationCenterRefundOrderGoods);
                } else {
                    // 福利商城退单商品存在
                    operationCenterRefundOrderGoods = operationCenterRefundOrderGoods.toBuilder()
                            .id(operationCenterRefundOrderGoodsDB.getId())
                            .updateUserId("0")
                            .updateTime(new Date())
                            .updateUserName("增量定时任务同步")
                            .build();
                    operationCenterRefundOrderGoodsUpdateList.add(operationCenterRefundOrderGoods);
                }
                // 如果有退单退运费商品则进行处理
                if (operationCenterRefundOrderGoodsDispatch != null) {
                    // 判断福利商城退单运费商品是否存在
                    OperationCenterOrderGoods operationCenterRefundOrderGoodsDispatchDB = operationCenterOrderGoodsMapper.selectOne(
                            Wrappers.<OperationCenterOrderGoods>lambdaQuery()
                                    .eq(OperationCenterOrderGoods::getOrderSn, flscShopOrderRefund.getRefundno())
                                    .eq(OperationCenterOrderGoods::getBizTypeId, OpCenterOrderBizTypeEnum.售后.getCode())
                                    .eq(OperationCenterOrderGoods::getOrderTypeCode, OpCenterOrderTypeEnum.福利商城.getCode())
                                    .eq(OperationCenterOrderGoods::getBizPlatformCode, OpCenterBizPlatformEnum.福利商城.getCode())
                                    .eq(OperationCenterOrderGoods::getOrderGoodsId, OpCenterOrderTypeEnum.福利商城.getCode() + "_" + OpCenterGoodsTypeEnum.运费.getPrefix() + flscShopOrder.getId())
                    );
                    if (operationCenterRefundOrderGoodsDispatchDB == null || operationCenterRefundOrderGoodsDispatchDB.getId() == null) {
                        // 福利商城退单运费商品不存在
                        operationCenterRefundOrderGoodsDispatch = operationCenterRefundOrderGoodsDispatch.toBuilder()
                                .createUserId("0")
                                .createUserName("增量定时任务同步")
                                .createTime(new Date())
                                .build();
                        operationCenterRefundOrderFreightGoodsInsertList.add(operationCenterRefundOrderGoodsDispatch);
                    } else {
                        // 福利商城退单运费商品存在
                        operationCenterRefundOrderGoodsDispatch = operationCenterRefundOrderGoodsDispatch.toBuilder()
                                .id(operationCenterRefundOrderGoodsDispatchDB.getId())
                                .updateUserId("0")
                                .updateTime(new Date())
                                .updateUserName("增量定时任务同步")
                                .build();
                        operationCenterRefundOrderFreightGoodsUpdateList.add(operationCenterRefundOrderGoodsDispatch);
                    }
                }


                /**
                 * 反写正单售后状态
                 */
                List<FlscShopOrderRefund> refundListForThisOrder = flscShopOrderRefundMapper.selectList(
                        Wrappers.<FlscShopOrderRefund>lambdaQuery()
                                .eq(FlscShopOrderRefund::getOrderid, flscShopOrder.getId())
                );
                if (CollectionUtil.isNotEmpty(refundListForThisOrder)) {
                    // 运营中心的正单
                    OperationCenterOrder operationCenterOrderExist = operationCenterOrderMapper.selectOne(
                            Wrappers.<OperationCenterOrder>lambdaQuery()
                                    .eq(OperationCenterOrder::getOrderSn, flscShopOrder.getOrdersn())
                                    .eq(OperationCenterOrder::getOrderBizTypeId, OpCenterOrderBizTypeEnum.订单.getCode())
                                    .eq(OperationCenterOrder::getOrderTypeCode, OpCenterOrderTypeEnum.福利商城.getCode())
                                    .eq(OperationCenterOrder::getBizPlatformCode, OpCenterBizPlatformEnum.福利商城.getCode())
                    );
                    if (operationCenterOrderExist != null && operationCenterOrderExist.getId() != null) {
                        // 如果有售后单，需要根据所有售后单状态判断 反写正单售后状态

                        // 全部取消
                        List<Integer> cancelOrRejected = new ArrayList<>();
                        cancelOrRejected.add(FlscAfterSaleStatusEnum.客户取消.getCode());
                        cancelOrRejected.add(FlscAfterSaleStatusEnum.已拒绝.getCode());


                        List<Integer> statusList = refundListForThisOrder.stream().map(FlscShopOrderRefund::getStatus).distinct().collect(Collectors.toList());
                        if (CollectionUtil.isNotEmpty(statusList) && statusList.size() > 0) {
                            if ((statusList.size() == 1 && statusList.contains(FlscAfterSaleStatusEnum.完成.getCode()))
                                    || (statusList.size() == 2 && statusList.contains(FlscAfterSaleStatusEnum.完成.getCode()) && statusList.contains(FlscAfterSaleStatusEnum.已拒绝.getCode()))
                                    || (statusList.size() == 2 && statusList.contains(FlscAfterSaleStatusEnum.完成.getCode()) && statusList.contains(FlscAfterSaleStatusEnum.客户取消.getCode()))) {
                                // 1. 全部完成 / 部分完成 部分取消/拒绝
                                operationCenterOrderExist.setPlatformAftersaleStatusId(OpCenterPlatformAftersaleStatusEnum.售后完成.getCode());
                            } else if (statusList.size() == 1 && statusList.contains(FlscAfterSaleStatusEnum.客户取消.getCode()) || statusList.size() == 1 && statusList.contains(FlscAfterSaleStatusEnum.已拒绝.getCode()) || statusList.size() == 2 && statusList.containsAll(cancelOrRejected)) {
                                // 2. 全部取消
                                operationCenterOrderExist.setPlatformAftersaleStatusId(OpCenterPlatformAftersaleStatusEnum.售后取消.getCode());
                            } else {
                                // 3. 全部售后中
                                // 4. 部分售后中
                                operationCenterOrderExist.setPlatformAftersaleStatusId(OpCenterPlatformAftersaleStatusEnum.售后中.getCode());
                            }
                            // 反写售后单状态
                            operationCenterOrderExist.setUpdateUserId("0");
                            operationCenterOrderExist.setUpdateUserName("增量同步定时任务反写正单售后状态");
                            operationCenterOrderMapper.updateById(operationCenterOrderExist);
                        }
                    }
                }
            }
        }

        if (CollectionUtil.isNotEmpty(operationCenterRefundOrderInsertList)) {
            // 循环插入福利商城退单
            for (OperationCenterOrder operationCenterRefundOrder : operationCenterRefundOrderInsertList) {
                try {
                    operationCenterOrderMapper.insert(operationCenterRefundOrder);
                } catch (Exception e) {
                    // 异常数据需要记录在表
                    ThreadUtil.execute(() -> {
                        operationDataSyncErrorRecordService.save(
                                OperationDataSyncErrorRecord.builder()
                                        .syncType(OperationDataSyncTypeEnum.订单.getName())
                                        .bizPlatformCode(OpCenterBizPlatformEnum.福利商城.getCode())
                                        .bizPlatformName(OpCenterBizPlatformEnum.福利商城.getName())
                                        .mainParam(String.valueOf(operationCenterRefundOrder.getBizOrderId()))
                                        .orderSn(operationCenterRefundOrder.getOrderSn())
                                        .syncTime(new Date())
                                        .errMsg("退单信息新增失败: " + JSON.toJSONString(e))
                                        .retentionData(JSON.toJSONString(operationCenterRefundOrder))
                                        .handleStatus(OperationDataSyncErrorHandleStatusEnum.未处理.getCode())
                                        .build()
                        );
                    });
                    log.info("福利商城退单插入失败，订单号：{}", operationCenterRefundOrder.getOrderSn());
                    errorOrderCount++;
                }
            }
        }
        if (CollectionUtil.isNotEmpty(operationCenterRefundOrderUpdateList)) {
            // 循环更新福利商城退单
            for (OperationCenterOrder operationCenterRefundOrder : operationCenterRefundOrderUpdateList) {
                try {
                    operationCenterOrderMapper.updateById(operationCenterRefundOrder);
                } catch (Exception e) {
                    // 异常数据需要记录在表
                    ThreadUtil.execute(() -> {
                        operationDataSyncErrorRecordService.save(
                                OperationDataSyncErrorRecord.builder()
                                        .syncType(OperationDataSyncTypeEnum.订单.getName())
                                        .bizPlatformCode(OpCenterBizPlatformEnum.福利商城.getCode())
                                        .bizPlatformName(OpCenterBizPlatformEnum.福利商城.getName())
                                        .mainParam(String.valueOf(operationCenterRefundOrder.getBizOrderId()))
                                        .orderSn(operationCenterRefundOrder.getOrderSn())
                                        .syncTime(new Date())
                                        .errMsg("退单信息修改失败: " + JSON.toJSONString(e))
                                        .retentionData(JSON.toJSONString(operationCenterRefundOrder))
                                        .handleStatus(OperationDataSyncErrorHandleStatusEnum.未处理.getCode())
                                        .build()
                        );
                    });
                    log.info("福利商城退单更新失败，订单号：{}", operationCenterRefundOrder.getOrderSn());
                    errorOrderCount++;
                }
            }
        }
        if (CollectionUtil.isNotEmpty(operationCenterRefundOrderGoodsInsertList)) {
            // 循环插入福利商城退单商品
            for (OperationCenterOrderGoods operationCenterRefundOrderGoods : operationCenterRefundOrderGoodsInsertList) {
                try {
                    operationCenterOrderGoodsMapper.insert(operationCenterRefundOrderGoods);
                } catch (Exception e) {
                    // 异常数据需要记录在表
                    ThreadUtil.execute(() -> {
                        operationDataSyncErrorRecordService.save(
                                OperationDataSyncErrorRecord.builder()
                                        .syncType(OperationDataSyncTypeEnum.订单商品.getName())
                                        .bizPlatformCode(OpCenterBizPlatformEnum.福利商城.getCode())
                                        .bizPlatformName(OpCenterBizPlatformEnum.福利商城.getName())
                                        .mainParam(String.valueOf(operationCenterRefundOrderGoods.getOrderGoodsId()))
                                        .orderSn(operationCenterRefundOrderGoods.getOrderSn())
                                        .orderGoodsId(operationCenterRefundOrderGoods.getOrderGoodsId())
                                        .syncTime(new Date())
                                        .errMsg("退单商品新增失败 :" + JSON.toJSONString(e))
                                        .retentionData(JSON.toJSONString(operationCenterRefundOrderGoods))
                                        .handleStatus(OperationDataSyncErrorHandleStatusEnum.未处理.getCode())
                                        .build()
                        );
                    });
                    log.info("福利商城退单商品插入失败，订单号：{}", operationCenterRefundOrderGoods.getOrderSn());
                    errorOrderGoodsCount++;
                }
            }
        }
        if (CollectionUtil.isNotEmpty(operationCenterRefundOrderGoodsUpdateList)) {
            // 循环更新福利商城退单商品
            for (OperationCenterOrderGoods operationCenterRefundOrderGoods : operationCenterRefundOrderGoodsUpdateList) {
                try {
                    operationCenterOrderGoodsMapper.updateById(operationCenterRefundOrderGoods);
                } catch (Exception e) {
                    // 异常数据需要记录在表
                    ThreadUtil.execute(() -> {
                        operationDataSyncErrorRecordService.save(
                                OperationDataSyncErrorRecord.builder()
                                        .syncType(OperationDataSyncTypeEnum.订单商品.getName())
                                        .bizPlatformCode(OpCenterBizPlatformEnum.福利商城.getCode())
                                        .bizPlatformName(OpCenterBizPlatformEnum.福利商城.getName())
                                        .mainParam(String.valueOf(operationCenterRefundOrderGoods.getOrderGoodsId()))
                                        .orderSn(operationCenterRefundOrderGoods.getOrderSn())
                                        .orderGoodsId(operationCenterRefundOrderGoods.getOrderGoodsId())
                                        .syncTime(new Date())
                                        .errMsg("退单商品修改失败 :" + JSON.toJSONString(e))
                                        .retentionData(JSON.toJSONString(operationCenterRefundOrderGoods))
                                        .handleStatus(OperationDataSyncErrorHandleStatusEnum.未处理.getCode())
                                        .build()
                        );
                    });
                    log.info("福利商城退单商品更新失败，订单号：{}", operationCenterRefundOrderGoods.getOrderSn());
                    errorOrderGoodsCount++;
                }
            }
        }
        if (CollectionUtil.isNotEmpty(operationCenterRefundOrderFreightGoodsInsertList)) {
            // 循环插入福利商城退单运费商品
            for (OperationCenterOrderGoods operationCenterRefundOrderGoods : operationCenterRefundOrderFreightGoodsInsertList) {
                try {
                    operationCenterOrderGoodsMapper.insert(operationCenterRefundOrderGoods);
                } catch (Exception e) {
                    // 异常数据需要记录在表
                    ThreadUtil.execute(() -> {
                        operationDataSyncErrorRecordService.save(
                                OperationDataSyncErrorRecord.builder()
                                        .syncType(OperationDataSyncTypeEnum.订单商品.getName())
                                        .bizPlatformCode(OpCenterBizPlatformEnum.福利商城.getCode())
                                        .bizPlatformName(OpCenterBizPlatformEnum.福利商城.getName())
                                        .mainParam(String.valueOf(operationCenterRefundOrderGoods.getOrderGoodsId()))
                                        .orderSn(operationCenterRefundOrderGoods.getOrderSn())
                                        .orderGoodsId(operationCenterRefundOrderGoods.getOrderGoodsId())
                                        .syncTime(new Date())
                                        .errMsg("退单运费商品新增失败 :" + JSON.toJSONString(e))
                                        .retentionData(JSON.toJSONString(operationCenterRefundOrderGoods))
                                        .handleStatus(OperationDataSyncErrorHandleStatusEnum.未处理.getCode())
                                        .build()
                        );
                    });
                    log.info("福利商城退单运费商品插入失败，订单号：{}", operationCenterRefundOrderGoods.getOrderSn());
                    errorOrderFreightCount++;
                }
            }
        }
        if (CollectionUtil.isNotEmpty(operationCenterRefundOrderFreightGoodsUpdateList)) {
            // 循环更新福利商城退单运费商品
            for (OperationCenterOrderGoods operationCenterRefundOrderGoods : operationCenterRefundOrderFreightGoodsUpdateList) {
                try {
                    operationCenterOrderGoodsMapper.updateById(operationCenterRefundOrderGoods);
                } catch (Exception e) {
                    // 异常数据需要记录在表
                    ThreadUtil.execute(() -> {
                        operationDataSyncErrorRecordService.save(
                                OperationDataSyncErrorRecord.builder()
                                        .syncType(OperationDataSyncTypeEnum.订单商品.getName())
                                        .bizPlatformCode(OpCenterBizPlatformEnum.福利商城.getCode())
                                        .bizPlatformName(OpCenterBizPlatformEnum.福利商城.getName())
                                        .mainParam(String.valueOf(operationCenterRefundOrderGoods.getOrderGoodsId()))
                                        .orderSn(operationCenterRefundOrderGoods.getOrderSn())
                                        .orderGoodsId(operationCenterRefundOrderGoods.getOrderGoodsId())
                                        .syncTime(new Date())
                                        .errMsg("退单运费商品修改失败 :" + JSON.toJSONString(e))
                                        .retentionData(JSON.toJSONString(operationCenterRefundOrderGoods))
                                        .handleStatus(OperationDataSyncErrorHandleStatusEnum.未处理.getCode())
                                        .build()
                        );
                    });
                    log.info("福利商城退单运费商品更新失败，订单号：{}", operationCenterRefundOrderGoods.getOrderSn());
                    errorOrderFreightCount++;
                }
            }
        }
    }

    /**
     * 根据同步类型获取退单列表
     *
     * @param syncType
     * @param syncParam
     * @return
     */
    private List<FlscShopOrderRefund> getRefundOrderListBySyncType(int syncType, Object syncParam) {
        switch (syncType) {
            case 2:
                // 按照唯一标识同步
                syncParam = String.valueOf(syncParam);
                return flscShopOrderRefundMapper.selectList(
                        Wrappers.<FlscShopOrderRefund>lambdaQuery()
                                .eq(FlscShopOrderRefund::getRefundno, syncParam)
                );
            default:
                // 默认同步 & 按照指定时间同步 通用此方法
                Date date = (Date) syncParam;
                return flscShopOrderRefundMapper.selectList(
                        Wrappers.<FlscShopOrderRefund>lambdaQuery()
                                .ge(FlscShopOrderRefund::getSyncTime, date)
                );
        }
    }


    /**
     * 根据同步类型获取退单列表 & 分批
     *
     * @param syncType
     * @param syncParam
     * @param offset
     * @param batchSize
     * @return
     */
    private List<FlscShopOrderRefund> getRefundOrderListBySyncTypeAndForBatch(int syncType, Object syncParam, int offset, int batchSize) {
        switch (syncType) {
            case 2:
                // 按照唯一标识同步
                syncParam = String.valueOf(syncParam);
                return flscShopOrderRefundMapper.selectList(
                        Wrappers.<FlscShopOrderRefund>lambdaQuery()
                                .eq(FlscShopOrderRefund::getRefundno, syncParam)
                                .last("limit " + offset + "," + batchSize)
                );
            default:
                // 默认同步 & 按照指定时间同步 通用此方法
                return flscShopOrderRefundMapper.selectList(
                        Wrappers.<FlscShopOrderRefund>lambdaQuery()
                                .ge(FlscShopOrderRefund::getSyncTime, syncParam)
                                .last("limit " + offset + "," + batchSize)
                );
        }
    }
}
