package cn.superdesk.data.extract.job.flsc;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.thread.ThreadUtil;
import cn.superdesk.data.extract.dto.JobParamDTO;
import cn.superdesk.data.extract.entity.flsc.FlscShopThirdOrderPay;
import cn.superdesk.data.extract.entity.opCenter.OperationCenterOrder;
import cn.superdesk.data.extract.entity.opCenter.OperationCenterOrderGoods;
import cn.superdesk.data.extract.entity.opCenter.OperationDataSyncErrorRecord;
import cn.superdesk.data.extract.enums.OpCenterJobSyncTypeEnum;
import cn.superdesk.data.extract.enums.opCenter.OpCenterBizPlatformEnum;
import cn.superdesk.data.extract.enums.opCenter.OpCenterOrderTypeEnum;
import cn.superdesk.data.extract.enums.opCenter.OperationDataSyncErrorHandleStatusEnum;
import cn.superdesk.data.extract.enums.opCenter.OperationDataSyncTypeEnum;
import cn.superdesk.data.extract.mapper.flsc.FlscShopThirdOrderPayMapper;
import cn.superdesk.data.extract.mapper.opCenter.OperationCenterOrderGoodsMapper;
import cn.superdesk.data.extract.mapper.opCenter.OperationCenterOrderMapper;
import cn.superdesk.data.extract.service.flsc.FlscShopThirdOrderPayService;
import cn.superdesk.data.extract.service.opCenter.OperationDataSyncErrorRecordService;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author xc
 * @description
 * @date 2023/3/24 14:20
 */
@Component
@Slf4j
@RequiredArgsConstructor
public class FlscThirdOrderSyncJob {

    private final FlscShopThirdOrderPayMapper flscShopThirdOrderPayMapper;
    private final FlscShopThirdOrderPayService flscShopThirdOrderPayService;
    private final OperationCenterOrderMapper operationCenterOrderMapper;
    private final OperationCenterOrderGoodsMapper operationCenterOrderGoodsMapper;
    private final OperationDataSyncErrorRecordService operationDataSyncErrorRecordService;


    @XxlJob("flscThirdOrderSyncJobHandler")
    @Transactional(rollbackFor = Exception.class)
    public void flscThirdOrderSync() {
        String startTime = DateUtil.now();
        String jobParam = XxlJobHelper.getJobParam();
        log.info("任务参数 : {}", jobParam);
        XxlJobHelper.log("任务参数 : {}", jobParam);
        //解析参数
        JobParamDTO jobParamDTO = null;

        //福利商城第三方(蜂助手）订单列表
        List<FlscShopThirdOrderPay> flscShopThirdOrderList = null;
        //变动数据总数
        Integer total = 0;
        //同步类型
        int syncType;
        //数据查询参数
        Object syncParam;
        if (StringUtils.isNotBlank(jobParam)) {
            try {
                jobParamDTO = JSON.parseObject(jobParam, JobParamDTO.class);
            } catch (Exception e) {
                log.info("任务参数解析异常", e);
                XxlJobHelper.log("任务参数解析异常", e);
                throw new RuntimeException("任务参数解析异常");
            }
            if (jobParamDTO == null || (StringUtils.isBlank(jobParamDTO.getMainParam()) && StringUtils.isBlank(jobParamDTO.getSyncTime()))) {
                log.info("任务参数解析失败 参数内容: {}", jobParam);
                XxlJobHelper.log("任务参数解析失败 参数内容: {}", jobParam);
                throw new RuntimeException("任务参数解析失败");
            }

            String syncTime = jobParamDTO.getSyncTime();
            String mainParam = jobParamDTO.getMainParam();
            if (StringUtils.isNotBlank(syncTime)) {
                //按照指定时间同步
                try {
                    syncParam = DateUtil.parse(syncTime, "yyyy-MM-dd HH:mm:ss");
                } catch (Exception e) {
                    log.info("日期参数解析失败 任务失败 参数内容: {}", jobParam);
                    XxlJobHelper.log("日期参数解析失败 任务失败 参数内容: {}", jobParam);
                    throw new RuntimeException("日期参数解析失败");
                }
                log.info("按照指定时间同步 参数 : {}", syncTime);
                //数据变化量
                total = flscShopThirdOrderPayMapper.selectCount(
                        Wrappers.<FlscShopThirdOrderPay>lambdaQuery()
                                .ge(FlscShopThirdOrderPay::getSyncTime, syncParam)
                                .eq(FlscShopThirdOrderPay::getSourceFrom, OpCenterOrderTypeEnum.卡劵商城.getCode())
                );
                syncType = OpCenterJobSyncTypeEnum.按照指定时间同步.getCode();
            } else {
                //数据变化量
                total = flscShopThirdOrderPayMapper.selectCount(
                        Wrappers.<FlscShopThirdOrderPay>lambdaQuery()
                                .eq(FlscShopThirdOrderPay::getOrdersn, mainParam)
                                .eq(FlscShopThirdOrderPay::getSourceFrom, OpCenterOrderTypeEnum.卡劵商城.getCode())
                );
                syncParam = mainParam;
                syncType = OpCenterJobSyncTypeEnum.按照唯一标识同步.getCode();
            }
        } else {
            //获取十五分钟前的Date信息
            DateTime fifteenMinutesAgo = DateUtil.offsetMinute(DateUtil.date(), -15);
            //数据变化量
            total = flscShopThirdOrderPayMapper.selectCount(
                    Wrappers.<FlscShopThirdOrderPay>lambdaQuery()
                            .ge(FlscShopThirdOrderPay::getSyncTime, fifteenMinutesAgo)
                            .eq(FlscShopThirdOrderPay::getSourceFrom, OpCenterOrderTypeEnum.卡劵商城.getCode())
            );
            syncParam = fifteenMinutesAgo;
            syncType = OpCenterJobSyncTypeEnum.默认同步.getCode();
        }

        int errorOrderCount = 0;
        int errorOrderGoodsCount = 0;
        int batchSize = 1000;

        if (total == 0) {
            log.info("蜂助手订单同步 无数据变化，任务结束");
            XxlJobHelper.log("蜂助手订单同步 无数据变化，任务结束");
            return;
        } else if (total <= 1000) {
            //数据变动量小于1000条
            flscShopThirdOrderList = getThirdOrderListBySyncType(syncType, syncParam);
            batchHandle(flscShopThirdOrderList, errorOrderCount, errorOrderGoodsCount);
        } else {
            //数据变动量大于1000条
            int batch = total / batchSize;
            int remainder = total % batchSize;
            int times = remainder == 0 ? batch : batch + 1;
            int offset = 0;
            for (int i = 0; i < times; i++) {
                flscShopThirdOrderList = getThirdOrderListBySyncTypeAndForBatch(syncType, syncParam, offset, batchSize);
                batchHandle(flscShopThirdOrderList, errorOrderCount, errorOrderGoodsCount);
                flscShopThirdOrderList = null;
                offset += batchSize;
            }
        }
        log.info("蜂助手订单同步 同步类型：{}, 同步参数: {}, 开始时间:{}， 结束时间: {} 本次同步订单数量：{}, 订单总失败记录数: {}, 订单商品总失败记录数: {}",
                OpCenterJobSyncTypeEnum.getDescByCode(syncType), jobParam, startTime, DateUtil.now(), total, errorOrderCount, errorOrderGoodsCount);
    }

    private void batchHandle(List<FlscShopThirdOrderPay> flscShopThirdOrderList, int errorOrderCount, int errorOrderGoodsCount) {
        //第三方订单待新增列表
        List<OperationCenterOrder> operationCenterOrderInsertList = new ArrayList<>();
        //第三方订单待修改列表
        List<OperationCenterOrder> operationCenterOrderUpdateList = new ArrayList<>();
        //第三方订单商品待新增列表
        List<OperationCenterOrderGoods> operationCenterOrderGoodsInsertList = new ArrayList<>();
        //第三方订单商品待修改列表
        List<OperationCenterOrderGoods> operationCenterOrderGoodsUpdateList = new ArrayList<>();


        if (CollectionUtil.isNotEmpty(flscShopThirdOrderList)) {
            for (FlscShopThirdOrderPay flscShopThirdOrderPay : flscShopThirdOrderList) {
                //根据福利商城第三方订单组装运营中心订单
                OperationCenterOrder operationCenterOrder = flscShopThirdOrderPayService.buildOpCenterOrderFromFlscThirdOrder(flscShopThirdOrderPay);
                //根据福利商城第三方订单组装运营中心订单商品
                OperationCenterOrderGoods operationCenterOrderGoods = flscShopThirdOrderPayService.buildOpCenterOrderGoodsFromFlscThirdOrder(flscShopThirdOrderPay);

                //查询运营中心订单是否存在
                OperationCenterOrder operationCenterOrderDB = operationCenterOrderMapper.selectOne(
                        Wrappers.<OperationCenterOrder>lambdaQuery()
                                .eq(OperationCenterOrder::getOrderSn, operationCenterOrder.getOrderSn())
                                .eq(OperationCenterOrder::getBizPlatformCode, OpCenterBizPlatformEnum.福利商城.getCode())
                                .eq(OperationCenterOrder::getOrderTypeCode, OpCenterOrderTypeEnum.卡劵商城.getCode())
                );
                if (operationCenterOrderDB == null || operationCenterOrderDB.getId() == null) {
                    //运营中心订单不存在，新增
                    operationCenterOrder = operationCenterOrder.toBuilder()
                            .createUserId("0")
                            .createUserName("增量定时任务同步")
                            .createTime(new Date())
                            .build();
                    operationCenterOrderInsertList.add(operationCenterOrder);
                } else {
                    //运营中心订单存在，修改
                    //订单
                    operationCenterOrder = operationCenterOrder.toBuilder()
                            .id(operationCenterOrderDB.getId())
                            .platformAftersaleStatusId(operationCenterOrderDB.getPlatformAftersaleStatusId())
                            .platformAftersaleType(operationCenterOrderDB.getPlatformAftersaleType())
                            .platformAftersaleStatusId(operationCenterOrderDB.getPlatformAftersaleStatusId())
                            .updateTime(new Date())
                            .updateUserId("0")
                            .updateUserName("增量定时任务同步")
                            .build();
                    operationCenterOrderUpdateList.add(operationCenterOrder);
                }

                //查询运营中心订单商品是否存在
                OperationCenterOrderGoods operationCenterOrderGoodsDB = operationCenterOrderGoodsMapper.selectOne(
                        Wrappers.<OperationCenterOrderGoods>lambdaQuery()
                                .eq(OperationCenterOrderGoods::getOrderSn, operationCenterOrder.getOrderSn())
                                .eq(OperationCenterOrderGoods::getBizPlatformCode, OpCenterBizPlatformEnum.福利商城.getCode())
                                .eq(OperationCenterOrderGoods::getOrderTypeCode, OpCenterOrderTypeEnum.卡劵商城.getCode())
                );
                if (operationCenterOrderGoodsDB == null || operationCenterOrderGoodsDB.getId() == null) {
                    //运营中心订单商品不存在，新增
                    operationCenterOrderGoods = operationCenterOrderGoods.toBuilder()
                            .createUserId("0")
                            .createUserName("增量定时任务同步")
                            .createTime(new Date())
                            .build();
                    operationCenterOrderGoodsInsertList.add(operationCenterOrderGoods);
                } else {
                    //运营中心订单商品存在，修改
                    operationCenterOrderGoods = operationCenterOrderGoods.toBuilder()
                            .id(operationCenterOrderGoodsDB.getId())
                            .updateTime(new Date())
                            .updateUserId("0")
                            .updateUserName("增量定时任务同步")
                            .build();
                    operationCenterOrderGoodsUpdateList.add(operationCenterOrderGoods);
                }
            }
        }

        if (CollectionUtil.isNotEmpty(operationCenterOrderInsertList)) {
            //循环新增运营中心订单
            for (OperationCenterOrder operationCenterOrder : operationCenterOrderInsertList) {
                try {
                    operationCenterOrderMapper.insert(operationCenterOrder);
                } catch (Exception e) {
                    //异常数据需要记录在表
                    ThreadUtil.execute(() -> {
                        operationDataSyncErrorRecordService.save(
                                OperationDataSyncErrorRecord.builder()
                                        .syncType(OperationDataSyncTypeEnum.订单.getName())
                                        .bizPlatformCode(OpCenterBizPlatformEnum.福利商城.getCode())
                                        .bizPlatformName(OpCenterBizPlatformEnum.福利商城.getName())
                                        .mainParam(String.valueOf(operationCenterOrder.getBizOrderId()))
                                        .orderSn(operationCenterOrder.getOrderSn())
                                        .syncTime(new Date())
                                        .errMsg("第三方订单信息新增失败: " + JSON.toJSONString(e))
                                        .retentionData(JSON.toJSONString(operationCenterOrder))
                                        .handleStatus(OperationDataSyncErrorHandleStatusEnum.未处理.getCode())
                                        .build()
                        );
                    });
                    log.info("福利商城第三方订单&订单商品增量同步任务执行失败, 新增运营中心订单失败, 订单信息: {}", operationCenterOrder);
                    errorOrderCount++;
                }
            }
        }
        if (CollectionUtil.isNotEmpty(operationCenterOrderUpdateList)) {
            //循环修改运营中心订单
            for (OperationCenterOrder operationCenterOrder : operationCenterOrderUpdateList) {
                try {
                    operationCenterOrderMapper.updateById(operationCenterOrder);
                } catch (Exception e) {
                    //异常数据需要记录在表
                    ThreadUtil.execute(() -> {
                        operationDataSyncErrorRecordService.save(
                                OperationDataSyncErrorRecord.builder()
                                        .syncType(OperationDataSyncTypeEnum.订单.getName())
                                        .bizPlatformCode(OpCenterBizPlatformEnum.福利商城.getCode())
                                        .bizPlatformName(OpCenterBizPlatformEnum.福利商城.getName())
                                        .mainParam(String.valueOf(operationCenterOrder.getBizOrderId()))
                                        .orderSn(operationCenterOrder.getOrderSn())
                                        .syncTime(new Date())
                                        .errMsg("第三方订单信息修改失败: " + JSON.toJSONString(e))
                                        .retentionData(JSON.toJSONString(operationCenterOrder))
                                        .handleStatus(OperationDataSyncErrorHandleStatusEnum.未处理.getCode())
                                        .build()
                        );
                    });
                    log.info("福利商城第三方订单&订单商品增量同步任务执行失败, 修改运营中心订单失败, 订单信息: {}", operationCenterOrder);
                    errorOrderCount++;
                }
            }
        }
        if (CollectionUtil.isNotEmpty(operationCenterOrderGoodsInsertList)) {
            //循环新增运营中心订单商品
            for (OperationCenterOrderGoods operationCenterOrderGoods : operationCenterOrderGoodsInsertList) {
                try {
                    operationCenterOrderGoodsMapper.insert(operationCenterOrderGoods);
                } catch (Exception e) {
                    //异常数据需要记录在表
                    ThreadUtil.execute(() -> {
                        operationDataSyncErrorRecordService.save(
                                OperationDataSyncErrorRecord.builder()
                                        .syncType(OperationDataSyncTypeEnum.订单商品.getName())
                                        .bizPlatformCode(OpCenterBizPlatformEnum.福利商城.getCode())
                                        .bizPlatformName(OpCenterBizPlatformEnum.福利商城.getName())
                                        .mainParam(String.valueOf(operationCenterOrderGoods.getOrderGoodsId()))
                                        .orderSn(operationCenterOrderGoods.getOrderSn())
                                        .orderGoodsId(String.valueOf(operationCenterOrderGoods.getOrderGoodsId()))
                                        .syncTime(new Date())
                                        .errMsg("第三方订单商品信息新增失败: " + JSON.toJSONString(e))
                                        .retentionData(JSON.toJSONString(operationCenterOrderGoods))
                                        .handleStatus(OperationDataSyncErrorHandleStatusEnum.未处理.getCode())
                                        .build()
                        );
                    });
                    log.info("福利商城第三方订单&订单商品增量同步任务执行失败, 新增运营中心订单商品失败, 订单商品信息: {}", operationCenterOrderGoods);
                    errorOrderGoodsCount++;
                }
            }
        }
        if (CollectionUtil.isNotEmpty(operationCenterOrderGoodsUpdateList)) {
            //循环修改运营中心订单商品
            for (OperationCenterOrderGoods operationCenterOrderGoods : operationCenterOrderGoodsUpdateList) {
                try {
                    operationCenterOrderGoodsMapper.updateById(operationCenterOrderGoods);
                } catch (Exception e) {
                    //异常数据需要记录在表
                    ThreadUtil.execute(() -> {
                        operationDataSyncErrorRecordService.save(
                                OperationDataSyncErrorRecord.builder()
                                        .syncType(OperationDataSyncTypeEnum.订单商品.getName())
                                        .bizPlatformCode(OpCenterBizPlatformEnum.福利商城.getCode())
                                        .bizPlatformName(OpCenterBizPlatformEnum.福利商城.getName())
                                        .mainParam(String.valueOf(operationCenterOrderGoods.getOrderGoodsId()))
                                        .orderSn(operationCenterOrderGoods.getOrderSn())
                                        .orderGoodsId(String.valueOf(operationCenterOrderGoods.getOrderGoodsId()))
                                        .syncTime(new Date())
                                        .errMsg("第三方订单商品信息修改失败: " + JSON.toJSONString(e))
                                        .retentionData(JSON.toJSONString(operationCenterOrderGoods))
                                        .handleStatus(OperationDataSyncErrorHandleStatusEnum.未处理.getCode())
                                        .build()
                        );
                    });
                    log.info("福利商城第三方订单&订单商品增量同步任务执行失败, 修改运营中心订单商品失败, 订单商品信息: {}", operationCenterOrderGoods);
                    errorOrderGoodsCount++;
                }
            }
        }
    }

    /**
     * 根据同步类型获取第三方订单列表
     *
     * @param syncType
     * @param syncParam
     * @return
     */
    private List<FlscShopThirdOrderPay> getThirdOrderListBySyncType(int syncType, Object syncParam) {
        switch (syncType) {
            case 2:
                //按照唯一标识同步
                return flscShopThirdOrderPayMapper.selectList(
                        Wrappers.<FlscShopThirdOrderPay>lambdaQuery()
                                .eq(FlscShopThirdOrderPay::getOrdersn, syncParam)
                                .eq(FlscShopThirdOrderPay::getSourceFrom, OpCenterOrderTypeEnum.卡劵商城.getCode())
                );
            default:
                //获取十五分钟内变化的数据，实际定时任务每十分钟执行一次
                return flscShopThirdOrderPayMapper.selectList(
                        Wrappers.<FlscShopThirdOrderPay>lambdaQuery()
                                .ge(FlscShopThirdOrderPay::getSyncTime, syncParam)
                                .eq(FlscShopThirdOrderPay::getSourceFrom, OpCenterOrderTypeEnum.卡劵商城.getCode())
                );
        }
    }

    /**
     * 根据同步类型获取第三方订单列表 & 批量
     *
     * @param syncType
     * @param syncParam
     * @param offset
     * @param batchSize
     * @return
     */
    private List<FlscShopThirdOrderPay> getThirdOrderListBySyncTypeAndForBatch(int syncType, Object syncParam, int offset, int batchSize) {
        switch (syncType) {
            case 2:
                //按照唯一标识同步
                return flscShopThirdOrderPayMapper.selectList(
                        Wrappers.<FlscShopThirdOrderPay>lambdaQuery()
                                .eq(FlscShopThirdOrderPay::getOrdersn, syncParam)
                                .eq(FlscShopThirdOrderPay::getSourceFrom, OpCenterOrderTypeEnum.卡劵商城.getCode())
                                .last("limit " + offset + "," + batchSize)
                );
            default:
                //获取十五分钟内变化的数据，实际定时任务每十分钟执行一次
                return flscShopThirdOrderPayMapper.selectList(
                        Wrappers.<FlscShopThirdOrderPay>lambdaQuery()
                                .ge(FlscShopThirdOrderPay::getSyncTime, syncParam)
                                .eq(FlscShopThirdOrderPay::getSourceFrom, OpCenterOrderTypeEnum.卡劵商城.getCode())
                                .last("limit " + offset + "," + batchSize)
                );
        }
    }
}
