package cn.superdesk.data.extract.service.impl.flsc;

import cn.superdesk.data.extract.entity.flsc.FlscShopCategory;
import cn.superdesk.data.extract.entity.flsc.FlscShopGoods;
import cn.superdesk.data.extract.entity.flsc.FlscShopOrder;
import cn.superdesk.data.extract.entity.flsc.FlscShopOrderGoods;
import cn.superdesk.data.extract.entity.opCenter.OperationCenterOrder;
import cn.superdesk.data.extract.entity.opCenter.OperationCenterOrderGoods;
import cn.superdesk.data.extract.enums.opCenter.*;
import cn.superdesk.data.extract.mapper.flsc.*;
import cn.superdesk.data.extract.mapper.opCenter.OperationCenterOrderMapper;
import cn.superdesk.data.extract.service.flsc.FlscShopOrderGoodsService;
import cn.superdesk.data.extract.service.opCenter.OperationCenterOperatorSettlementStrategyService;
import cn.superdesk.data.extract.service.opCenter.OperationCenterOrderGoodsService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author storyxc
 * @since 2023-03-17
 */
@Service
@RequiredArgsConstructor
@Slf4j
public class FlscShopOrderGoodsServiceImpl extends ServiceImpl<FlscShopOrderGoodsMapper, FlscShopOrderGoods> implements FlscShopOrderGoodsService {
    private final FlscShopOrderGoodsMapper shopOrderGoodsMapper;
    private final FlscShopOrderMapper flscShopOrderMapper;
    private final OperationCenterOrderGoodsService operationCenterOrderGoodsService;
    private final OperationCenterOrderMapper operationCenterOrderMapper;
    private final FlscShopGoodsMapper flscShopGoodsMapper;
    private final OperationCenterOperatorSettlementStrategyService operationCenterOperatorSettlementStrategyService;
    private final FlscShopCategoryMapper flscShopCategoryMapper;

    /**
     * 全量处理福利商城订单商品
     */
    @Override
    @Async
    @Transactional(rollbackFor = Exception.class)
    public void handleAllOrderGoods() {
        Integer count = shopOrderGoodsMapper.selectTotalCount();
        log.info("count =========== {}", count);

        int batchSize = 1000;
        int mod = count % batchSize;
        int time = mod > 0 ? count / batchSize + 1 : count / batchSize;
        log.info("总条数 ===== {}", count);
        log.info("处理批次数 ===== {}", time);
        // 成功处理记录数
        int successCount = 0;

        int offset = 0;
        List<FlscShopOrderGoods> list = null;
        List<OperationCenterOrderGoods> insertOrderGoodsList = null;
        for (int i = 0; i < time; i++) {
            list = shopOrderGoodsMapper.selectByBatch(offset, batchSize);
            insertOrderGoodsList = new ArrayList<>();
            for (FlscShopOrderGoods orderGoods : list) {
                /**
                 * 归属订单
                 */
                FlscShopOrder flscShopOrder = flscShopOrderMapper.selectById(orderGoods.getOrderid());
                if (flscShopOrder == null || flscShopOrder.getId() == null) {
                    log.info("订单商品 id : {}, 关联订单id : {} 不存在", orderGoods.getId(), orderGoods.getOrderid());
                    continue;
                }
                OperationCenterOrder operationCenterOrder = operationCenterOrderMapper.selectOne(Wrappers.<OperationCenterOrder>lambdaQuery()
                        .eq(OperationCenterOrder::getOrderSn, flscShopOrder.getOrdersn())
                        .eq(OperationCenterOrder::getBizPlatformCode, OpCenterBizPlatformEnum.福利商城.getCode())
                        .eq(OperationCenterOrder::getOrderTypeCode, OpCenterOrderTypeEnum.福利商城.getCode())
                        .eq(OperationCenterOrder::getOrderBizTypeId, OpCenterOrderBizTypeEnum.订单.getCode()));
                if (operationCenterOrder == null || operationCenterOrder.getId() == null) {
                    log.info("订单商品 id : {}, 关联订单id : {} 运营中心订单不存在", orderGoods.getId(), orderGoods.getOrderid());
                    continue;
                }
                /**
                 * 关联的商品 shop_goods
                 */
                FlscShopGoods goods = flscShopGoodsMapper.selectById(orderGoods.getGoodsid());
                if (goods == null || goods.getId() == null) {
                    log.info("订单商品 id : {}, 关联商品id : {} 不存在", orderGoods.getId(), orderGoods.getGoodsid());
                    continue;
                }
                OperationCenterOrderGoods operationCenterOrderGoods = buildOpCenterOrderGoodsFlscOrderGoods(orderGoods, flscShopOrder, operationCenterOrder, goods);
                operationCenterOrderGoods = operationCenterOrderGoods.toBuilder()
                        .createUserId("0")// 创建人
                        .createUserName("全量处理福利商城订单商品")
                        .createTime(new Date())
                        .build();
                insertOrderGoodsList.add(operationCenterOrderGoods);

            }
            operationCenterOrderGoodsService.saveBatch(insertOrderGoodsList);

            successCount += insertOrderGoodsList.size();
            insertOrderGoodsList = null;
            offset += batchSize;
            list = null;
        }
        log.info("=======================全量处理福利商城正单商品数据完成，总记录数： {}, 成功处理记录数： {}=======================", count, successCount);
    }

    @Override
    public OperationCenterOrderGoods buildOpCenterOrderGoodsFlscOrderGoods(FlscShopOrderGoods flscShopOrderGoods, FlscShopOrder flscShopOrder, OperationCenterOrder operationCenterOrder, FlscShopGoods goods) {

        /**
         * 商品金额部分
         */
        // 商品数量
        Integer total = flscShopOrderGoods.getTotal();
        BigDecimal decimalTotal = new BigDecimal(total);
        // 商品原单价
        BigDecimal originalUnitPrice = flscShopOrderGoods.getProductprice();
        // 商品原总价 = 商品原单价 * 商品数量
        BigDecimal originalTotalPrice = originalUnitPrice.multiply(decimalTotal);
        // 商品销售单价
        BigDecimal sellingUnitPrice = flscShopOrderGoods.getMarketPrice();
        // 商品销售总价
        BigDecimal sellingTotalPrice = sellingUnitPrice.multiply(decimalTotal);
        // 商品协议单价
        BigDecimal negotiatedUnitPrice = flscShopOrderGoods.getCostpriceReal();
        // 商品协议总价
        BigDecimal negotiatedTotalPrice = flscShopOrderGoods.getCostpriceReal().multiply(decimalTotal);
        // 商品成本单价
        BigDecimal costUnitPrice = flscShopOrderGoods.getCostprice();
        // 商品成本总价
        BigDecimal costTotalPrice = costUnitPrice.multiply(decimalTotal);
        // 订单商品金额（商品实付）
        BigDecimal orderGoodsAmount = flscShopOrderGoods.getNewRealPriceTotal();
        // 优惠金额 = 商品销售总价 - 商品订单金额
        BigDecimal discountAmount = sellingTotalPrice.subtract(orderGoodsAmount);


        /**
         * 结算价
         */
        BigDecimal settlementAmount = operationCenterOperatorSettlementStrategyService.getOrderGoodsSettlementAmount(operationCenterOrder.getOperatorId(), OpCenterBizPlatformEnum.福利商城.getCode(), sellingTotalPrice, negotiatedTotalPrice);
        BigDecimal settlementUnitPrice = operationCenterOperatorSettlementStrategyService.getOrderGoodsSettlementUnitPrice(operationCenterOrder.getOperatorId(), OpCenterBizPlatformEnum.福利商城.getCode(), sellingUnitPrice, negotiatedUnitPrice);
        /**
         * 商品分类
         */
        String cateLevel1Id = null;
        String cateLevel1Name = null;
        String cateLevel2Id = null;
        String cateLevel2Name = null;
        String cateLevel3Id = null;
        String cateLevel3Name = null;
        FlscShopCategory cate1 = flscShopCategoryMapper.selectById(goods.getPcate());
        if (cate1 != null && cate1.getId() != null) {
            cateLevel1Id = String.valueOf(cate1.getId());
            cateLevel1Name = cate1.getName();
        }
        FlscShopCategory cate2 = flscShopCategoryMapper.selectById(goods.getCcate());
        if (cate2 != null && cate2.getId() != null) {
            cateLevel2Id = String.valueOf(cate2.getId());
            cateLevel2Name = cate2.getName();
        }
        FlscShopCategory cate3 = flscShopCategoryMapper.selectById(goods.getTcate());
        if (cate3 != null && cate3.getId() != null) {
            cateLevel3Id = String.valueOf(cate3.getId());
            cateLevel3Name = cate3.getName();
        }

        return OperationCenterOrderGoods.builder()
                .bizTypeId(OpCenterOrderBizTypeEnum.订单.getCode())// 业务类型id
                .orderGoodsId(String.valueOf(flscShopOrderGoods.getId()))// 订单商品id
                .orderSn(flscShopOrder.getOrdersn())// 订单编号
                .orderId(String.valueOf(flscShopOrder.getId()))// 订单id
                .parentOrderId(operationCenterOrder.getParentOrderId())// 父订单id，取订单的
                .parentOrderSn(operationCenterOrder.getParentOrderSn())// 父订单sn，取订单的
                .thirdPartyOrderSn(operationCenterOrder.getThirdPartyOrderSn())// 第三方单号
                .bizPlatformCode(operationCenterOrder.getBizPlatformCode())// 业务平台编码
                .bizPlatformName(operationCenterOrder.getBizPlatformName())// 业务平台名称
                .orderTypeCode(operationCenterOrder.getOrderTypeCode())// 订单类型编码
                .orderTypeName(operationCenterOrder.getOrderTypeName())// 订单类型名称
                .orderCreateTime(operationCenterOrder.getOrderCreateTime())// 订单时间
                .finishTime(operationCenterOrder.getFinishTime())// 完成时间
                .isPackage(OpCenterIsPackageEnum.非套餐.getCode())// 是否套餐
                .packageId(null)// 套餐id
                .packageName(null)// 套餐名称
                .goodsTypeId(OpCenterGoodsTypeEnum.商品.getCode())// 商品类型id
                .goodsId(String.valueOf(flscShopOrderGoods.getGoodsid()))// 商品id
                .goodsSku(flscShopOrderGoods.getGoodssn())// 商品sku
                .goodsName(goods.getTitle())// 商品名称
                .quantity(total)// 商品数量
                .unit(StringUtils.isBlank(goods.getUnit()) ? "项" : goods.getUnit())// 商品单位
                .originalUnitPrice(originalUnitPrice)// 商品原单价
                .originalTotalPrice(originalTotalPrice)// 商品原总价
                .sellingTotalPrice(sellingTotalPrice)// 商品销售总价
                .sellingUnitPrice(sellingUnitPrice)// 商品销售单价
                .negotiatedTotalPrice(negotiatedTotalPrice)// 商品协议总价
                .negotiatedUnitPrice(negotiatedUnitPrice)// 商品协议单价
                .settlementAmount(settlementAmount)// 结算总价
                .settlementUnitPrice(settlementUnitPrice)// 结算单价
                .costUnitPrice(costUnitPrice)// 成本单价
                .costTotalPrice(costTotalPrice)// 成本总价
                .discountAmount(discountAmount)// 优惠金额 先置空 等业务方给方案
                .platformTaxCode(flscShopOrderGoods.getOperationTaxCode())// 平台税务编码
                .platformTaxRate(flscShopOrderGoods.getOperationTaxRate())// 平台税率
                .merchantTaxCode(flscShopOrderGoods.getMerchTaxCode())// 商户税务编码
                .merchantTaxRate(flscShopOrderGoods.getMerchTaxRate())// 商户税率
                .isDutyFree(OpCenterIsDutyFreeEnum.不免税.getCode())// 默认不免税
                .operatorId(operationCenterOrder.getOperatorId())// 运营商id
                .operatorName(operationCenterOrder.getOperatorName())// 运营商名称
                .supplierId(operationCenterOrder.getSupplierId())// 供应商id
                .supplierName(operationCenterOrder.getSupplierName())// 供应商名称
                .brandId(null)// 品牌id 预留 暂时不管
                .brandName(null)// 品牌名称 预留 暂时不管
                .cateLevel1Id(cateLevel1Id)// 商品一级分类id
                .cateLevel1Name(cateLevel1Name)// 商品一级分类名称
                .cateLevel2Id(cateLevel2Id)// 商品二级分类id
                .cateLevel2Name(cateLevel2Name)// 商品二级分类名称
                .cateLevel3Id(cateLevel3Id)// 商品三级分类id
                .cateLevel3Name(cateLevel3Name)// 商品三级分类名称
                .goodsSpecification(goods.getSelfOption())// 商品型号/规格
                .otherAttribute(null)// 其他属性
                .refundOriginOrderSn(null)// 退单原单号 正单商品为空
                .build();
    }
}
