package cn.superdesk.data.extract.service.impl.flsc;

import cn.hutool.core.collection.CollectionUtil;
import cn.superdesk.data.extract.entity.flsc.*;
import cn.superdesk.data.extract.entity.opCenter.OperationCenterOrder;
import cn.superdesk.data.extract.entity.opCenter.OperationCenterOrderGoods;
import cn.superdesk.data.extract.entity.opCenter.OperationCenterPayment;
import cn.superdesk.data.extract.enums.flsc.FlscAfterSaleStatusEnum;
import cn.superdesk.data.extract.enums.flsc.FlscAfterSaleTypeEnum;
import cn.superdesk.data.extract.enums.flsc.FlscPayTypeEnum;
import cn.superdesk.data.extract.enums.opCenter.*;
import cn.superdesk.data.extract.mapper.flsc.*;
import cn.superdesk.data.extract.mapper.opCenter.OperationCenterOrderMapper;
import cn.superdesk.data.extract.service.flsc.FlscShopOrderRefundService;
import cn.superdesk.data.extract.service.opCenter.OperationCenterOperatorSettlementStrategyService;
import cn.superdesk.data.extract.service.opCenter.OperationCenterOrderGoodsService;
import cn.superdesk.data.extract.service.opCenter.OperationCenterOrderService;
import cn.superdesk.data.extract.service.opCenter.OperationCenterPaymentBizService;
import cn.superdesk.data.extract.utils.PhpDeserializeUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author storyxc
 * @since 2023-03-17
 */
@Service
@RequiredArgsConstructor
@Slf4j
public class FlscShopOrderRefundServiceImpl extends ServiceImpl<FlscShopOrderRefundMapper, FlscShopOrderRefund> implements FlscShopOrderRefundService {
    private final FlscShopOrderRefundMapper flscShopOrderRefundMapper;
    private final FlscShopOrderMapper flscShopOrderMapper;
    private final FlscShopOperatorsUserMapper flscShopOperatorsUserMapper;
    private final FlscShopEnterpriseUserMapper flscShopEnterpriseUserMapper;
    private final FlscShopMemberMapper flscShopMemberMapper;
    private final FlscShopMerchUserMapper flscShopMerchUserMapper;
    private final FlscShopOrderGoodsMapper flscShopOrderGoodsMapper;
    private final OperationCenterOperatorSettlementStrategyService operationCenterOperatorSettlementStrategyService;
    private final OperationCenterPaymentBizService operationCenterPaymentBizService;
    private final OperationCenterOrderService operationCenterOrderService;
    private final OperationCenterOrderGoodsService operationCenterOrderGoodsService;
    private final FlscShopGoodsMapper flscShopGoodsMapper;
    private final FlscShopCategoryMapper flscShopCategoryMapper;
    private final OperationCenterOrderMapper operationCenterOrderMapper;

    /**
     * 全量处理福利商城退单
     */
    @Override
    @Async
    @Transactional(rollbackFor = Exception.class)
    public void handleAllRefundOrder() {
        Integer count = flscShopOrderRefundMapper.selectTotalCount();
        int batchSize = 1000;
        int mod = count % batchSize;
        int time = mod > 0 ? count / batchSize + 1 : count / batchSize;
        log.info("总条数 ===== {}", count);
        log.info("处理批次数 ===== {}", time);

        // 成功处理的条数
        int successCount = 0;

        int offset = 0;
        List<FlscShopOrderRefund> list = null;
        List<OperationCenterOrder> insertList = null;
        List<OperationCenterOrderGoods> insertRefundOrderGoodsList = null;
        List<OperationCenterOrderGoods> insertRefundOrderFreightList = null;
        for (int i = 0; i < time; i++) {
            list = flscShopOrderRefundMapper.selectByBatch(offset, batchSize);
            insertList = new ArrayList<>();
            insertRefundOrderGoodsList = new ArrayList<>();
            insertRefundOrderFreightList = new ArrayList<>();
            for (FlscShopOrderRefund flscShopOrderRefund : list) {
                /**
                 * 对应正单的主键id 如果小于0 是商城手动维护的退单，不需要处理
                 */
                Integer orderId = flscShopOrderRefund.getOrderid();
                if (orderId < 0) {
                    continue;
                }
                FlscShopOrder flscShopOrder = flscShopOrderMapper.selectById(flscShopOrderRefund.getOrderid());
                OperationCenterOrder operationCenterOrderRefund = buildOpCenterOrderFromFlscRefundOrder(flscShopOrderRefund, flscShopOrder);
                operationCenterOrderRefund = operationCenterOrderRefund.toBuilder()
                        .createUserId("0")
                        .createUserName("全量处理退单")
                        .createTime(new Date())
                        .build();
                insertList.add(operationCenterOrderRefund);


                // 构造退单商品

                /**
                 * 如果有关联的订单商品，需要构造退单商品
                 */
                OperationCenterOrderGoods operationCenterRefundOrderGoods = buildOperationCenterRefundOrderGoods(flscShopOrderRefund, operationCenterOrderRefund, flscShopOrder);
                if (operationCenterRefundOrderGoods != null) {
                    operationCenterRefundOrderGoods = operationCenterRefundOrderGoods.toBuilder()
                            .createUserId("0")
                            .createUserName("全量处理退单商品")
                            .createTime(new Date())
                            .build();
                    insertRefundOrderGoodsList.add(operationCenterRefundOrderGoods);
                }

                // 构造福利商城退运费商品
                OperationCenterOrderGoods operationCenterRefundOrderGoodsDispatch = null;
                if (BigDecimal.ZERO.compareTo(flscShopOrderRefund.getRefundDispatchprice()) < 0) {
                    operationCenterRefundOrderGoodsDispatch = buildOperationCenterRefundOrderGoodsDispatch(flscShopOrderRefund, operationCenterOrderRefund, flscShopOrder);
                    insertRefundOrderFreightList.add(operationCenterRefundOrderGoodsDispatch);
                }


                /**
                 * 反写正单信息
                 */
                List<FlscShopOrderRefund> refundListForThisOrder = flscShopOrderRefundMapper.selectList(
                        Wrappers.<FlscShopOrderRefund>lambdaQuery()
                                .eq(FlscShopOrderRefund::getOrderid, flscShopOrder.getId())
                );
                if (CollectionUtil.isNotEmpty(refundListForThisOrder)) {
                    // 运营中心的正单
                    OperationCenterOrder operationCenterOrderExist = operationCenterOrderMapper.selectOne(
                            Wrappers.<OperationCenterOrder>lambdaQuery()
                                    .eq(OperationCenterOrder::getOrderSn, flscShopOrder.getOrdersn())
                                    .eq(OperationCenterOrder::getOrderBizTypeId, OpCenterOrderBizTypeEnum.订单.getCode())
                                    .eq(OperationCenterOrder::getOrderTypeCode, OpCenterOrderTypeEnum.福利商城.getCode())
                                    .eq(OperationCenterOrder::getBizPlatformCode, OpCenterBizPlatformEnum.福利商城.getCode())
                    );
                    if (operationCenterOrderExist != null && operationCenterOrderExist.getId() != null) {
                        // 如果有售后单，需要根据所有售后单状态判断 反写正单售后状态

                        // 全部取消
                        List<Integer> cancelOrRejected = new ArrayList<>();
                        cancelOrRejected.add(FlscAfterSaleStatusEnum.客户取消.getCode());
                        cancelOrRejected.add(FlscAfterSaleStatusEnum.已拒绝.getCode());


                        List<Integer> statusList = refundListForThisOrder.stream().map(FlscShopOrderRefund::getStatus).distinct().collect(Collectors.toList());
                        if (CollectionUtil.isNotEmpty(statusList) && statusList.size() > 0) {
                            if ((statusList.size() == 1 && statusList.contains(FlscAfterSaleStatusEnum.完成.getCode()))
                                    || (statusList.size() == 2 && statusList.contains(FlscAfterSaleStatusEnum.完成.getCode()) && statusList.contains(FlscAfterSaleStatusEnum.已拒绝.getCode()))
                                    || (statusList.size() == 2 && statusList.contains(FlscAfterSaleStatusEnum.完成.getCode()) && statusList.contains(FlscAfterSaleStatusEnum.客户取消.getCode()))) {
                                // 1. 全部完成 / 部分完成 部分取消/拒绝
                                operationCenterOrderExist.setPlatformAftersaleStatusId(OpCenterPlatformAftersaleStatusEnum.售后完成.getCode());
                            } else if (statusList.size() == 1 && statusList.contains(FlscAfterSaleStatusEnum.客户取消.getCode()) || statusList.size() == 1 && statusList.contains(FlscAfterSaleStatusEnum.已拒绝.getCode()) || statusList.size() == 2 && statusList.containsAll(cancelOrRejected)) {
                                // 2. 全部取消
                                operationCenterOrderExist.setPlatformAftersaleStatusId(OpCenterPlatformAftersaleStatusEnum.售后取消.getCode());
                            } else {
                                // 3. 全部售后中
                                // 4. 部分售后中
                                operationCenterOrderExist.setPlatformAftersaleStatusId(OpCenterPlatformAftersaleStatusEnum.售后中.getCode());
                            }
                            // 反写售后单状态
                            operationCenterOrderExist.setUpdateUserId("0");
                            operationCenterOrderExist.setUpdateUserName("全量同步定时任务反写正单售后状态");
                            operationCenterOrderMapper.updateById(operationCenterOrderExist);
                        }
                    }
                }

            }

            operationCenterOrderService.saveBatch(insertList);
            if (CollectionUtil.isNotEmpty(insertRefundOrderGoodsList)) {
                operationCenterOrderGoodsService.saveBatch(insertRefundOrderGoodsList);
            }
            if (CollectionUtil.isNotEmpty(insertRefundOrderFreightList)) {
                operationCenterOrderGoodsService.saveBatch(insertRefundOrderFreightList);
            }
            successCount += insertList.size();
            insertList = null;
            insertRefundOrderGoodsList = null;
            insertRefundOrderFreightList = null;
            offset += batchSize;
            list = null;
        }
        log.info("=======================全量处理福利商城退单完成,总记录数：{}, 成功处理记录数 :{}=======================", count, successCount);
    }

    /**
     * 根据福利商城退单构建运营中心退单数据
     *
     * @param flscShopOrderRefund
     * @return
     */
    @Override
    public OperationCenterOrder buildOpCenterOrderFromFlscRefundOrder(FlscShopOrderRefund flscShopOrderRefund, FlscShopOrder flscShopOrder) {


        /**
         * 福利商城运营商id
         */
        Integer operatorId = flscShopOrder.getOperatorsId();
        String operatorName = "";
        if (operatorId != null && operatorId != 0) {
            FlscShopOperatorsUser flscShopOperatorsUser = flscShopOperatorsUserMapper.selectById(operatorId);
            if (flscShopOperatorsUser != null) {
                operatorName = flscShopOperatorsUser.getOperatorsName();
            }
        }
        /**
         * 福利商城项目id
         */
        Integer organizationId = flscShopOrder.getMemberOrganizationId();
        String organizationName = "";
        /**
         * 福利商城企业id
         */
        Integer memberEnterpriseId = flscShopOrder.getMemberEnterpriseId();
        if (memberEnterpriseId != null && memberEnterpriseId != 0) {
            FlscShopEnterpriseUser flscShopEnterpriseUser = flscShopEnterpriseUserMapper.selectById(memberEnterpriseId);
            if (flscShopEnterpriseUser != null) {
                organizationName = StringUtils.isBlank(flscShopEnterpriseUser.getOrganName()) ? "" : flscShopEnterpriseUser.getOrganName();
            }
        }
        /**
         * 福利商城用户openid
         */
        String openid = flscShopOrder.getOpenid();
        String buyerName = null;
        String buyerDeptName = null;
        String buyerMobile = null;
        FlscShopMember flscShopMember = flscShopMemberMapper.selectOne(Wrappers.<FlscShopMember>lambdaQuery().eq(FlscShopMember::getOpenid, openid));
        if (flscShopMember != null) {
            buyerName = flscShopMember.getRealname();
            buyerDeptName = flscShopMember.getDepartmentName();
            buyerMobile = flscShopMember.getMobile();
        }
        /**
         * 福利商城供应商id
         */
        Integer merchId = flscShopOrderRefund.getMerchid();
        String supplierName = "";
        FlscShopMerchUser flscShopMerchUser = flscShopMerchUserMapper.selectById(merchId);
        if (flscShopMerchUser != null) {
            supplierName = flscShopMerchUser.getMerchname();
        }
        /**
         * 售后单金额相关
         */
        // 退商品数量
        BigDecimal refundTotalDecimal = new BigDecimal(flscShopOrderRefund.getRefundTotal());
        // 售后退运费金额
        BigDecimal refundDispatchprice = flscShopOrderRefund.getRefundDispatchprice();
        // 订单商品金额 不含运费
        BigDecimal orderGoodsAmount = BigDecimal.ZERO;
        // 订单协议金额
        BigDecimal orderNegotiatedAmount = BigDecimal.ZERO;
        // 订单成本金额
        BigDecimal orderCostPrice = BigDecimal.ZERO;
        // 订单金额（实付） 售后单取申请退款金额
        BigDecimal orderPrice = BigDecimal.ZERO;
        // 售后运费
        BigDecimal dispatchPrice = BigDecimal.ZERO;
        // 售后商户承担运费
        BigDecimal merchantTransportAmount = BigDecimal.ZERO;
        // 优惠金额
        BigDecimal discountAmount = BigDecimal.ZERO;

        // 换货/维修 售后单不需要计算金额
        if (!FlscAfterSaleTypeEnum.换货.getCode().equals(flscShopOrderRefund.getRtype()) && !FlscAfterSaleTypeEnum.维修.getCode().equals(flscShopOrderRefund.getRtype())) {
            List<FlscShopOrderGoods> flscShopOrderGoods = flscShopOrderGoodsMapper.selectList(Wrappers.<FlscShopOrderGoods>lambdaQuery().eq(FlscShopOrderGoods::getId, flscShopOrderRefund.getOrderGoodsId()));
            if (CollectionUtil.isNotEmpty(flscShopOrderGoods)) {
                // 订单商品金额 不含运费
                orderGoodsAmount = flscShopOrderGoods.stream().map(g -> g.getMarketPrice().multiply(refundTotalDecimal)).reduce(BigDecimal.ZERO, BigDecimal::add).negate();
                // 订单协议金额 含用户承担运费
                orderNegotiatedAmount = flscShopOrderGoods.stream().map(order -> order.getCostpriceReal().multiply(refundTotalDecimal)).reduce(refundDispatchprice, BigDecimal::add).negate();
                // 订单成本金额 含商户承担运费（商城没有退商户承担运费数据 所以不需要加）
                orderCostPrice = flscShopOrderGoods.stream().map(order -> order.getCostprice().multiply(refundTotalDecimal)).reduce(BigDecimal.ZERO, BigDecimal::add).negate();
                // 订单金额（实付） 售后单取申请退款金额
                orderPrice = flscShopOrderRefund.getApplyprice().negate();// 退单取实退金额，需要为负数
                // 售后运费 取退单表数据，需要为负数
                dispatchPrice = refundDispatchprice.negate();//
                // merchantTransportAmount = BigDecimal.ZERO;//售后商户承担运费
                // 优惠金额 = 所有商品 [ (销售总价 - 订单商品金额) * (退商品数量 / 订单商品数量) ] 四舍五入保留2位小数 的累加，
                discountAmount = flscShopOrderGoods.stream().map(g -> {
                    BigDecimal marketPrice = g.getMarketPrice();// 销售单价
                    BigDecimal decimalTotal = new BigDecimal(g.getTotal());
                    BigDecimal sellingTotalPrice = marketPrice.multiply(decimalTotal);
                    return sellingTotalPrice.subtract(g.getNewRealPriceTotal()).multiply(refundTotalDecimal).divide(decimalTotal, 2, RoundingMode.HALF_UP);
                }).reduce(BigDecimal.ZERO, BigDecimal::add).negate();
            }
        }

        /**
         * 订单结算金额
         */
        BigDecimal orderSettlementAmount = operationCenterOperatorSettlementStrategyService.getOrderSettlementAmount(operatorId, OpCenterBizPlatformEnum.福利商城.getCode(), orderPrice, orderNegotiatedAmount);

        /**
         * 平台支付方式
         */
        OperationCenterPayment platformPayment = operationCenterPaymentBizService.getPlatformPayment(OpCenterBizPlatformEnum.福利商城.getCode(), flscShopOrder.getPaytype());

        /**
         * 退款状态
         */
        Integer refundPayStatus;
        if (FlscAfterSaleTypeEnum.退货退款.getCode().equals(flscShopOrderRefund.getRtype())) {
            if (FlscAfterSaleStatusEnum.完成.getCode().equals(flscShopOrderRefund.getStatus())) {
                refundPayStatus = OpCenterOrderPayStatusEnum.已退款.getCode();
            } else {
                refundPayStatus = OpCenterOrderPayStatusEnum.未退款.getCode();
            }
        } else {
            // 包括 维修/换货/退货
            refundPayStatus = OpCenterOrderPayStatusEnum.未退款.getCode();
        }


        /**
         * 反序列化address获取收货信息
         * { "area": "双流区", "jd_vop_town_code": "71576", "address": "测试", "town": "怡心街道", "city": "成都市", "openid": "wap_user_17_15008155867_30991", "mobile": "15525252525", "label": "家", "jd_vop_county_code": "49324", "uniacid": "17", "realname": "张三", "zipcode": "", "deleted": "0", "jd_vop_province_code": "22", "province": "四川", "jd_vop_area": "22_1930_49324_71576", "jd_vop_city_code": "1930", "id": "9485", "isdefault": "1" }
         */
        String address = flscShopOrder.getAddress();
        HashMap<String, String> deserializedMap = PhpDeserializeUtil.getDeserialized(address);
        String consignee = StringUtils.isBlank(deserializedMap.get("realname")) ? "" : deserializedMap.get("realname");
        String consigneeMobile = StringUtils.isBlank(deserializedMap.get("mobile")) ? "" : deserializedMap.get("mobile");
        String consigneeAddressId = StringUtils.isBlank(deserializedMap.get("id")) ? "" : deserializedMap.get("id");
        String province = StringUtils.isBlank(deserializedMap.get("province")) ? "" : deserializedMap.get("province");
        String city = StringUtils.isBlank(deserializedMap.get("city")) ? "" : deserializedMap.get("city");
        String county = StringUtils.isBlank(deserializedMap.get("area")) ? "" : deserializedMap.get("area");
        String street = StringUtils.isBlank(deserializedMap.get("town")) ? "" : deserializedMap.get("town");
        String addressDetail = StringUtils.isBlank(deserializedMap.get("address")) ? "" : deserializedMap.get("address");


        return OperationCenterOrder.builder().orderBizTypeId(OpCenterOrderBizTypeEnum.售后.getCode())// 业务类型id
                .orderSn(flscShopOrderRefund.getRefundno())// 订单编号 售后单取售后单号
                .bizOrderId(String.valueOf(flscShopOrderRefund.getId()))// 订单id
                .parentOrderSn(null)// 父订单编号 售后单为空
                .parentOrderId(null)// 父订单id 售后单为空
                .orderDataTypeId(OpCenterOrderDataTypeEnum.原始订单.getCode())// 订单数据类型id
                .thirdPartyOrderSn(flscShopOrderRefund.getThirdrefundno())// 三方单号 第三方售后编号
                .bizPlatformCode(OpCenterBizPlatformEnum.福利商城.getCode())// 业务平台编码
                .bizPlatformName(OpCenterBizPlatformEnum.福利商城.getName())// 业务平台名称
                .orderTypeCode(OpCenterOrderTypeEnum.福利商城.getCode())// 订单类型编码
                .orderTypeName(OpCenterOrderTypeEnum.福利商城.getName())// 订单类型名称
                .bizSourceId(OpCenterBizSourceEnum.福利商城.getCode())// 业务来源id
                .bizSourceName(OpCenterBizSourceEnum.福利商城.getDesc())// 业务来源名称
                .bizTerminalId(OpCenterBizTerminalEnum.微信小程序.getCode())// 业务终端id
                .bizTerminalCode(null)// 业务终端编码，福利商城置空
                .bizTerminalName(null)// 业务终端名称，福利商城置空
                .orderCreateTime(flscShopOrderRefund.getCreatetime() > 0 ? new Date(flscShopOrderRefund.getCreatetime() * 1000L) : null)// 订单时间 售后单的创建时间
                .tradingTime(flscShopOrderRefund.getRefundtime() > 0 ? new Date(flscShopOrderRefund.getRefundtime() * 1000L) : null)// 订单支付/退款时间
                .sendTime(flscShopOrderRefund.getSendtime() > 0 ? new Date(flscShopOrderRefund.getSendtime() * 1000L) : null)// 发货时间
                .finishTime(flscShopOrderRefund.getEndtime() > 0 ? new Date(flscShopOrderRefund.getEndtime() * 1000L) : null)// 完成时间
                .operatorId(String.valueOf(operatorId))// 运营商id（业务方）
                .operatorName(operatorName)// 运营商名称（业务方） 福利商城从ims_superdesk_shop_operators_user表里取
                .orgId(organizationId == null ? null : String.valueOf(organizationId))// 项目id
                .orgName(organizationName)// 项目名称
                .enterpriseId(memberEnterpriseId == null ? null : String.valueOf(memberEnterpriseId))// 企业id
                .enterpriseName(flscShopOrder.getMemberEnterpriseName())// 企业名称
                .buyerId(openid)// 下单人userId 福利商城取openid
                .buyerName(buyerName)// 下单人姓名
                .buyerDeptId(null)// 下单人部门id 福利商城没有
                .buyerDeptName(buyerDeptName)// 下单人部门名称
                .buyerMobile(buyerMobile)// 下单人手机号
                .supplierId(String.valueOf(merchId))// 供应商id
                .supplierName(supplierName)// 供应商名称
                .orderGoodsAmount(orderGoodsAmount)// 订单所有商品销售总价之和，不含运费 售后单为负数
                .orderNegotiatedAmount(orderNegotiatedAmount)// 订单协议金额 售后单为负数
                .orderPrice(orderPrice)// 订单金额（实付金额）  售后单取退款金额
                .orderSettlementAmount(orderSettlementAmount)// 订单结算金额
                .orderCostPrice(orderCostPrice)// 订单成本金额
                .transportAmount(dispatchPrice)// 运费
                .merchantTransportAmount(merchantTransportAmount)// 商户承担运费 只有京东订单有
                .discountAmount(discountAmount)// 优惠金额
                .couponId(null)// 优惠券id
                .discountDesc(null)// 优惠券描述
                .platformAftersaleStatusId(FlscAfterSaleStatusEnum.getOpStatusIdByBizStatusId(flscShopOrderRefund.getStatus()))// 平台售后状态id
                .platformAftersaleTypeId(FlscAfterSaleTypeEnum.getOpAfterSaleTypeIdByBizId(flscShopOrderRefund.getRtype()))// 平台售后类型id
                .platformAftersaleType(FlscAfterSaleTypeEnum.getOpAfterSaleTypeNameByBizId(flscShopOrderRefund.getRtype()))// 平台售后类型
                .bizOrderPayTypeId(String.valueOf(flscShopOrder.getPaytype()))// 业务订单支付方式id
                .bizOrderPayType(FlscPayTypeEnum.getNameById(flscShopOrder.getPaytype()))// 业务订单支付方式
                .platformPayTypeId(platformPayment.getId())// 平台支付方式id
                .platformPayType(platformPayment.getName())// 平台支付方式名称
                .orderPayStatus(refundPayStatus)// 订单支付状态
                .bizOrderStatusId(String.valueOf(flscShopOrderRefund.getStatus()))// 业务订单状态id
                .bizOrderStatusName(FlscAfterSaleStatusEnum.getBizOrderStatusDescByBizId(flscShopOrderRefund.getStatus()))// 业务订单状态名称
                .transactionStatusId(FlscAfterSaleStatusEnum.getOpTransactionIdByBizId(flscShopOrderRefund.getStatus()))// 交易状态id
                .orderRemark(flscShopOrderRefund.getContent())// 订单备注
                .merchantRemark(flscShopOrderRefund.getReply())// 商家备注
                .consignee(consignee)// 收货人
                .consigneeMobile(consigneeMobile)// 收货人电话
                .consigneeAddress(province + city + county + street + addressDetail).consigneeAddressId(consigneeAddressId)// 收货地址id
                .province(province)// 省
                .city(city)// 市
                .county(county)// 区/县
                .street(street)// 街道
                .addressDetail(addressDetail).distributionTypeId(null)// 配送方式id
                .distributionType(null)// 配送方式名称
                .expressCode(flscShopOrderRefund.getRexpress())// 物流快递编码
                .expressName(flscShopOrderRefund.getRexpresscom())// 物流快递名称
                .expressTrackingNo(flscShopOrderRefund.getRexpresssn())// 物流快递单号
                .refundOriginOrderSn(flscShopOrder.getOrdersn())// 退单原单号
                .deleted(0).build();
    }


    /**
     * 根据福利商城退单构建运营中心退单商品数据
     *
     * @param flscShopOrderRefund        福利商城退单
     * @param operationCenterRefundOrder 运营中心退单
     * @param flscShopOrder              福利商城订单
     * @return
     */
    @Override
    public OperationCenterOrderGoods buildOperationCenterRefundOrderGoods(FlscShopOrderRefund flscShopOrderRefund, OperationCenterOrder operationCenterRefundOrder, FlscShopOrder flscShopOrder) {
        if (flscShopOrderRefund.getOrderGoodsId() != null && flscShopOrderRefund.getOrderGoodsId() != 0) {
            FlscShopOrderGoods orderGoods = flscShopOrderGoodsMapper.selectById(flscShopOrderRefund.getOrderGoodsId());

            FlscShopOperatorsUser flscShopOperatorsUser = flscShopOperatorsUserMapper.selectById(flscShopOrder.getOperatorsId());
            if (orderGoods != null) {
                /**
                 * 关联的商品 shop_goods
                 */
                FlscShopGoods goods = flscShopGoodsMapper.selectById(orderGoods.getGoodsid());

                if (goods != null) {
                    /**
                     * 退货商品数量
                     */
                    Integer refundTotal = flscShopOrderRefund.getRefundTotal();
                    BigDecimal decimalRefundTotal = new BigDecimal(refundTotal);

                    BigDecimal originalUnitPrice = BigDecimal.ZERO;// 商品原单价
                    BigDecimal sellingUnitPrice = BigDecimal.ZERO;// 商品销售单价，商城是三位小数，运营中心保留两位小数会自动四舍五入
                    BigDecimal negotiatedUnitPrice = BigDecimal.ZERO;// 商品协议单价
                    BigDecimal costUnitPrice = BigDecimal.ZERO;// 成本单价

                    BigDecimal refundOriginalTotalPrice = BigDecimal.ZERO;// 退款商品原总价
                    BigDecimal refundSellingTotalPrice = BigDecimal.ZERO;// 退款商品原销售总价
                    BigDecimal refundNegotiatedTotalPrice = BigDecimal.ZERO;// 退款商品原协议总价
                    BigDecimal refundCostTotalPrice = BigDecimal.ZERO;// 退款商品原成本总价
                    BigDecimal refundSettlementAmount = BigDecimal.ZERO;// 结算金额
                    BigDecimal settlementUnitPrice = BigDecimal.ZERO;// 结算单价

                    BigDecimal orderGoodsAmount = BigDecimal.ZERO;// 商品订单金额 商品实退金额
                    BigDecimal discountAmount = BigDecimal.ZERO;// 优惠金额

                    if (!FlscAfterSaleTypeEnum.换货.getCode().equals(flscShopOrderRefund.getRtype()) && !FlscAfterSaleTypeEnum.维修.getCode().equals(flscShopOrderRefund.getRtype())) {
                        refundTotal = Math.negateExact(refundTotal);
                        /**
                         * 单价
                         */
                        // 商品原单价
                        originalUnitPrice = orderGoods.getProductprice();
                        // 商品销售单价
                        sellingUnitPrice = orderGoods.getMarketPrice();
                        // 商品协议单价
                        negotiatedUnitPrice = orderGoods.getCostpriceReal();
                        // 商品成本单价
                        costUnitPrice = orderGoods.getCostprice();

                        /**
                         * 总价计算
                         */
                        // 退款商品原总价
                        refundOriginalTotalPrice = originalUnitPrice.multiply(decimalRefundTotal).negate();
                        // 退款商品销售总价
                        refundSellingTotalPrice = sellingUnitPrice.multiply(decimalRefundTotal).negate();
                        // 退款商品协议总价
                        refundNegotiatedTotalPrice = negotiatedUnitPrice.multiply(decimalRefundTotal).negate();
                        // 退款商品成本总价
                        refundCostTotalPrice = costUnitPrice.multiply(decimalRefundTotal).negate();
                        // 商品订单金额（商品实退金额 不含运费） 转换为负数 = 全部退款金额 - 运费
                        orderGoodsAmount = (flscShopOrderRefund.getApplyprice().subtract(flscShopOrderRefund.getRefundDispatchprice())).negate();
                        // 优惠金额 = (销售总价 - 订单商品金额) * (退商品数量 / 订单商品数量)  四舍五入保留2位小数
                        BigDecimal decimalTotal = new BigDecimal(orderGoods.getTotal());
                        BigDecimal sellingTotalPrice = sellingUnitPrice.multiply(decimalTotal);
                        discountAmount = sellingTotalPrice.subtract(orderGoods.getNewRealPriceTotal()).multiply(decimalRefundTotal).divide(decimalTotal, 2, RoundingMode.HALF_UP).negate();

                        /**
                         * 结算总价
                         */
                        refundSettlementAmount = operationCenterOperatorSettlementStrategyService.getOrderGoodsSettlementAmount(operationCenterRefundOrder.getOperatorId(), OpCenterBizPlatformEnum.福利商城.getCode(), refundSellingTotalPrice, refundNegotiatedTotalPrice);
                        settlementUnitPrice = operationCenterOperatorSettlementStrategyService.getOrderGoodsSettlementUnitPrice(operationCenterRefundOrder.getOperatorId(), OpCenterBizPlatformEnum.福利商城.getCode(), sellingUnitPrice, negotiatedUnitPrice);

                    }


                    /**
                     * 商品分类
                     */
                    String cateLevel1Id = null;
                    String cateLevel1Name = null;
                    String cateLevel2Id = null;
                    String cateLevel2Name = null;
                    String cateLevel3Id = null;
                    String cateLevel3Name = null;
                    FlscShopCategory cate1 = flscShopCategoryMapper.selectById(goods.getPcate());
                    if (cate1 != null && cate1.getId() != null) {
                        cateLevel1Id = String.valueOf(cate1.getId());
                        cateLevel1Name = cate1.getName();
                    }
                    FlscShopCategory cate2 = flscShopCategoryMapper.selectById(goods.getCcate());
                    if (cate2 != null && cate2.getId() != null) {
                        cateLevel2Id = String.valueOf(cate2.getId());
                        cateLevel2Name = cate2.getName();
                    }
                    FlscShopCategory cate3 = flscShopCategoryMapper.selectById(goods.getTcate());
                    if (cate3 != null && cate3.getId() != null) {
                        cateLevel3Id = String.valueOf(cate3.getId());
                        cateLevel3Name = cate3.getName();
                    }

                    String operatorId = null;
                    String operatorName = null;
                    if (flscShopOperatorsUser != null) {
                        operatorId = String.valueOf(flscShopOperatorsUser.getId());
                        operatorName = flscShopOperatorsUser.getOperatorsName();
                    }

                    return OperationCenterOrderGoods.builder().bizTypeId(OpCenterOrderBizTypeEnum.售后.getCode())// 业务类型id
                            .orderGoodsId(String.valueOf(orderGoods.getId()))// 订单商品id
                            .orderSn(flscShopOrderRefund.getRefundno())// 订单编号 退单的
                            .orderId(String.valueOf(flscShopOrderRefund.getId()))// 订单id
                            .parentOrderId(null)// 父订单id 售后单没有
                            .parentOrderSn(null)// 父订单sn 售后单没有
                            .thirdPartyOrderSn(flscShopOrderRefund.getThirdrefundno())// 第三方单号
                            .bizPlatformCode(operationCenterRefundOrder.getBizPlatformCode())// 业务平台编码
                            .bizPlatformName(operationCenterRefundOrder.getBizPlatformName())// 业务平台名称
                            .orderTypeCode(operationCenterRefundOrder.getOrderTypeCode())// 订单类型编码
                            .orderTypeName(operationCenterRefundOrder.getOrderTypeName())// 订单类型名称
                            .orderCreateTime(operationCenterRefundOrder.getOrderCreateTime())// 订单时间
                            .finishTime(operationCenterRefundOrder.getFinishTime())// 完成时间
                            .isPackage(OpCenterIsPackageEnum.非套餐.getCode())// 是否套餐
                            .packageId(null)// 套餐id
                            .packageName(null)// 套餐名称
                            .goodsTypeId(OpCenterGoodsTypeEnum.商品.getCode())// 商品类型id
                            .goodsId(String.valueOf(orderGoods.getGoodsid()))// 商品id
                            .goodsSku(orderGoods.getGoodssn())// 商品sku
                            .goodsName(goods.getTitle())// 商品名称
                            .quantity(refundTotal)// 商品数量 负数
                            .unit(StringUtils.isBlank(goods.getUnit()) ? "项" : goods.getUnit())// 商品单位
                            .originalUnitPrice(originalUnitPrice)// 商品原单价
                            .originalTotalPrice(refundOriginalTotalPrice)// 商品原总价
                            .sellingTotalPrice(refundSellingTotalPrice)// 商品销售总价
                            .sellingUnitPrice(sellingUnitPrice)// 商品销售单价
                            .negotiatedTotalPrice(refundNegotiatedTotalPrice)// 商品协议总价
                            .negotiatedUnitPrice(negotiatedUnitPrice)// 商品协议单价
                            .settlementAmount(refundSettlementAmount)// 结算总价
                            .settlementUnitPrice(settlementUnitPrice)// 结算单价
                            .costUnitPrice(costUnitPrice)// 成本单价
                            .costTotalPrice(refundCostTotalPrice)// 成本总价
                            .orderGoodsAmount(orderGoodsAmount)// 商品订单金额（实退金额 不含运费） 转换为负数
                            .discountAmount(discountAmount)// 优惠金额
                            .platformTaxCode(orderGoods.getOperationTaxCode())// 平台税务编码
                            .platformTaxRate(orderGoods.getOperationTaxRate())// 平台税率
                            .merchantTaxCode(orderGoods.getMerchTaxCode())// 商户税务编码
                            .merchantTaxRate(orderGoods.getMerchTaxRate())// 商户税率
                            .isDutyFree(OpCenterIsDutyFreeEnum.不免税.getCode())// 默认不免税
                            .operatorId(operatorId)// 运营商id
                            .operatorName(operatorName)// 运营商名称
                            .supplierId(operationCenterRefundOrder.getSupplierId())// 供应商id
                            .supplierName(operationCenterRefundOrder.getSupplierName())// 供应商名称
                            .brandId(null)// 品牌id 预留 暂时不管
                            .brandName(null)// 品牌名称 预留 暂时不管
                            .cateLevel1Id(cateLevel1Id)// 商品一级分类id
                            .cateLevel1Name(cateLevel1Name)// 商品一级分类名称
                            .cateLevel2Id(cateLevel2Id)// 商品二级分类id
                            .cateLevel2Name(cateLevel2Name)// 商品二级分类名称
                            .cateLevel3Id(cateLevel3Id)// 商品三级分类id
                            .cateLevel3Name(cateLevel3Name)// 商品三级分类名称
                            .goodsSpecification(goods.getSelfOption())// 商品型号/规格
                            .otherAttribute(null)// 其他属性
                            .refundOriginOrderSn(flscShopOrder.getOrdersn())// 退单原单号 退单商品取正单sn
                            .build();
                }
            }
        }
        return null;
    }

    /**
     * 构建运营中心退单退运费商品
     *
     * @param flscShopOrderRefund
     * @param operationCenterRefundOrder
     * @param flscShopOrder
     * @return
     */
    @Override
    public OperationCenterOrderGoods buildOperationCenterRefundOrderGoodsDispatch(FlscShopOrderRefund flscShopOrderRefund, OperationCenterOrder operationCenterRefundOrder, FlscShopOrder flscShopOrder) {
        BigDecimal refundDispatchprice = flscShopOrderRefund.getRefundDispatchprice();
        return OperationCenterOrderGoods.builder()
                .bizTypeId(OpCenterOrderBizTypeEnum.售后.getCode())// 业务类型id
                .orderGoodsId(OpCenterOrderTypeEnum.福利商城.getCode() + "_" + OpCenterGoodsTypeEnum.运费.getPrefix() + flscShopOrderRefund.getId())// 退单退运费商品id 运费id构造：FLSC_YF退单id
                .orderSn(flscShopOrder.getOrdersn())// 订单编号
                .orderId(String.valueOf(flscShopOrder.getId()))// 订单id
                .parentOrderId(null)// 售后单没有
                .parentOrderSn(null)// 售后单没有
                .thirdPartyOrderSn(flscShopOrderRefund.getThirdrefundno())// 第三方单号
                .bizPlatformCode(operationCenterRefundOrder.getBizPlatformCode())// 业务平台编码
                .bizPlatformName(operationCenterRefundOrder.getBizPlatformName())// 业务平台名称
                .orderTypeCode(operationCenterRefundOrder.getOrderTypeCode())// 订单类型编码
                .orderTypeName(operationCenterRefundOrder.getOrderTypeName())// 订单类型名称
                .orderCreateTime(operationCenterRefundOrder.getOrderCreateTime())// 订单时间
                .finishTime(operationCenterRefundOrder.getFinishTime())// 完成时间
                .isPackage(OpCenterIsPackageEnum.非套餐.getCode())// 是否套餐
                .packageId(null)// 套餐id
                .packageName(null)// 套餐名称
                .goodsTypeId(OpCenterGoodsTypeEnum.运费.getCode())// 商品类型id
                .goodsId(String.valueOf(OpCenterCustomizeGoodsEnum.运费.getCode()))// 商品id
                .goodsSku(null)// 商品sku
                .goodsName(OpCenterGoodsTypeEnum.运费.getDesc())// 商品名称
                .quantity(1)// 商品数量
                .unit("项")// 商品单位
                .originalUnitPrice(refundDispatchprice)// 商品原单价
                .originalTotalPrice(refundDispatchprice)// 商品原总价
                .sellingTotalPrice(refundDispatchprice)// 商品销售总价
                .sellingUnitPrice(refundDispatchprice)// 商品销售单价
                .negotiatedTotalPrice(refundDispatchprice)// 商品协议总价
                .negotiatedUnitPrice(refundDispatchprice)// 商品协议单价
                .orderGoodsAmount(refundDispatchprice)// 订单商品总价 20230425新增字段
                .settlementAmount(refundDispatchprice)// 结算总价
                .settlementUnitPrice(refundDispatchprice)// 结算单价
                .costUnitPrice(refundDispatchprice)// 成本单价
                .costTotalPrice(refundDispatchprice)// 成本总价
                .discountAmount(BigDecimal.ZERO)// 优惠金额
                .platformTaxCode(null)// 平台税务编码
                .platformTaxRate(BigDecimal.ZERO)// 平台税率
                .merchantTaxCode(null)// 商户税务编码
                .merchantTaxRate(BigDecimal.ZERO)// 商户税率
                .isDutyFree(OpCenterIsDutyFreeEnum.不免税.getCode())// 默认不免税
                .operatorId(operationCenterRefundOrder.getOperatorId())// 运营商id
                .operatorName(operationCenterRefundOrder.getOperatorName())// 运营商名称
                .supplierId(operationCenterRefundOrder.getSupplierId())// 供应商id
                .supplierName(operationCenterRefundOrder.getSupplierName())// 供应商名称
                .brandId(null)// 品牌id 预留 暂时不管
                .brandName(null)// 品牌名称 预留 暂时不管
                .cateLevel1Id(null)// 商品一级分类id
                .cateLevel1Name(null)// 商品一级分类名称
                .cateLevel2Id(null)// 商品二级分类id
                .cateLevel2Name(null)// 商品二级分类名称
                .cateLevel3Id(null)// 商品三级分类id
                .cateLevel3Name(null)// 商品三级分类名称
                .goodsSpecification(null)// 商品型号/规格
                .otherAttribute(null)// 其他属性
                .refundOriginOrderSn(flscShopOrder.getOrdersn())// 退单原正单sn
                .build();
    }
}
