package cn.superdesk.data.extract.service.impl.flsc;

import cn.hutool.core.collection.CollectionUtil;
import cn.superdesk.data.extract.entity.flsc.*;
import cn.superdesk.data.extract.entity.opCenter.OperationCenterOrder;
import cn.superdesk.data.extract.entity.opCenter.OperationCenterOrderGoods;
import cn.superdesk.data.extract.entity.opCenter.OperationCenterPayment;
import cn.superdesk.data.extract.enums.flsc.FlscPayTypeEnum;
import cn.superdesk.data.extract.enums.flsc.FlscThirdOrderPayStatusEnum;
import cn.superdesk.data.extract.enums.opCenter.*;
import cn.superdesk.data.extract.mapper.flsc.*;
import cn.superdesk.data.extract.mapper.opCenter.OperationCenterOrderMapper;
import cn.superdesk.data.extract.service.flsc.FlscShopThirdOrderPayService;
import cn.superdesk.data.extract.service.opCenter.OperationCenterOrderGoodsService;
import cn.superdesk.data.extract.service.opCenter.OperationCenterOrderService;
import cn.superdesk.data.extract.service.opCenter.OperationCenterPaymentBizService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author storyxc
 * @since 2023-03-17
 */
@Service
@RequiredArgsConstructor
@Slf4j
public class FlscShopThirdOrderPayServiceImpl extends ServiceImpl<FlscShopThirdOrderPayMapper, FlscShopThirdOrderPay> implements FlscShopThirdOrderPayService {
    private final FlscShopThirdOrderPayMapper flscShopThirdOrderPayMapper;
    private final OperationCenterOrderService operationCenterOrderService;
    private final FlscShopOperatorsUserMapper flscShopOperatorsUserMapper;
    private final FlscShopEnterpriseUserMapper flscShopEnterpriseUserMapper;
    private final FlscShopMemberMapper flscShopMemberMapper;
    private final FlscShopMerchUserMapper flscShopMerchUserMapper;
    private final OperationCenterPaymentBizService operationCenterPaymentBizService;
    private final OperationCenterOrderMapper operationCenterOrderMapper;
    private final OperationCenterOrderGoodsService operationCenterOrderGoodsService;
    private final FlscShopThirdOrderRefundMapper flscShopThirdOrderRefundMapper;

    @Async
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void handleAllOrder() {
        Integer count = flscShopThirdOrderPayMapper.selectTotalCount();
        int batchSize = 1000;
        int mod = count % batchSize;
        int time = mod > 0 ? count / batchSize + 1 : count / batchSize;
        log.info("总条数 ===== {}", count);
        log.info("处理批次数 ===== {}", time);

        //成功插入记录数
        int successCount = 0;

        int offset = 0;
        List<FlscShopThirdOrderPay> list = null;
        List<OperationCenterOrder> insertList = null;
        for (int i = 0; i < time; i++) {
            list = flscShopThirdOrderPayMapper.selectByBatch(offset, batchSize);
            insertList = new ArrayList<>();
            for (FlscShopThirdOrderPay flscShopThirdOrderPay : list) {
                //组装第三方正单
                OperationCenterOrder opCenterOrder = buildOpCenterOrderFromFlscThirdOrder(flscShopThirdOrderPay);
                opCenterOrder = opCenterOrder.toBuilder()
                        .createUserId("0")
                        .createUserName("全量处理第三方正单")
                        .createTime(new Date())
                        .updateUserId(null)
                        .updateTime(null)
                        .updateUserName(null)
                        .build();
                insertList.add(opCenterOrder);
            }
            successCount += insertList.size();
            operationCenterOrderService.saveBatch(insertList);
            insertList = null;
            offset += batchSize;
            list = null;
        }
        log.info("=======================全量处理第三方正单数据完成, 总记录数: {}, 成功插入记录数: {}=======================", count, successCount);
    }


    @Override
    @Async
    @Transactional(rollbackFor = Exception.class)
    public void handleAllThirdOrderGoods() {
        log.info("=======================全量处理第三方订单商品数据开始=======================");
        // 处理第三方正单数据，读取第三方正单中蜂助手的订单并构造商品数据
        Integer count = flscShopThirdOrderPayMapper.selectTotalCount();

        int batchSize = 1000;
        int mod = count % batchSize;
        int time = mod > 0 ? count / batchSize + 1 : count / batchSize;
        log.info("总条数 ===== {}", count);
        log.info("处理批次数 ===== {}", time);

        //成功插入记录数
        int successCount = 0;

        int offset = 0;
        List<FlscShopThirdOrderPay> list = null;
        List<OperationCenterOrderGoods> insertList = null;
        for (int i = 0; i < time; i++) {
            list = flscShopThirdOrderPayMapper.selectByBatch(offset, batchSize);
            insertList = new ArrayList<>();
            for (FlscShopThirdOrderPay flscShopThirdOrderPay : list) {
                //组装第三方正单商品
                OperationCenterOrderGoods operationCenterOrderGoods = buildOpCenterOrderGoodsFromFlscThirdOrder(flscShopThirdOrderPay);
                operationCenterOrderGoods = operationCenterOrderGoods.toBuilder()
                        .createUserId("0")
                        .createUserName("全量处理第三方订单商品数据")
                        .createTime(new Date())
                        .updateTime(null)
                        .updateUserId(null)
                        .updateUserName(null)
                        .build();
                insertList.add(operationCenterOrderGoods);
            }
            successCount += insertList.size();
            operationCenterOrderGoodsService.saveBatch(insertList);
            insertList = null;
            offset += batchSize;
            list = null;
        }
        log.info("=======================全量处理第三方订单商品数据完成=======================");


        log.info("=======================全量处理第三方退单商品数据开始=======================");
        // 处理第三方退单数据，读取第三方退单中蜂助手的退单并构造商品数据
        Integer count2 = flscShopThirdOrderRefundMapper.selectTotalCount();
        log.info("第三方退单总条数 ===== {}", count2);
        int time2 = count2 % batchSize > 0 ? count2 / batchSize + 1 : count2 / batchSize;
        log.info("第三方退单处理批次数 ===== {}", time2);
        int offset2 = 0;

        //成功插入记录数
        int successCount2 = 0;


        List<FlscShopThirdOrderRefund> list2 = null;
        List<OperationCenterOrderGoods> insertList2 = null;

        for (int i = 0; i < time2; i++) {
            list2 = flscShopThirdOrderRefundMapper.selectByBatch(offset2, batchSize);
            insertList2 = new ArrayList<>();

            for (FlscShopThirdOrderRefund flscShopThirdOrderRefund : list2) {

                FlscShopThirdOrderPay flscShopThirdOrderPay = flscShopThirdOrderPayMapper.selectById(flscShopThirdOrderRefund.getOrderid());
                if (flscShopThirdOrderPay == null) {
                    log.info("第三方退单数据中蜂助手的退单未找到对应的订单数据，退单编号：{}", flscShopThirdOrderRefund.getRefundno());
                    continue;
                }
                OperationCenterOrder operationCenterOrder = operationCenterOrderMapper.selectOne(
                        Wrappers.<OperationCenterOrder>lambdaQuery()
                                .eq(OperationCenterOrder::getOrderSn, flscShopThirdOrderRefund.getRefundno())
                                .eq(OperationCenterOrder::getOrderBizTypeId, OpCenterOrderBizTypeEnum.售后.getCode())
                                .eq(OperationCenterOrder::getOrderTypeCode, OpCenterOrderTypeEnum.卡劵商城.getCode())
                                .eq(OperationCenterOrder::getBizPlatformCode, OpCenterBizPlatformEnum.福利商城.getCode())
                );
                if (operationCenterOrder == null) {
                    log.info("第三方退单数据中蜂助手的退单未找到对应的订单数据，退单编号：{}", flscShopThirdOrderRefund.getRefundno());
                    continue;
                }
                //组装第三方退单商品
                OperationCenterOrderGoods thirdRefundOrderGoods = buildOpCenterRefundOrderGoodsFromFlscThirdOrder(flscShopThirdOrderRefund, flscShopThirdOrderPay);
                thirdRefundOrderGoods = thirdRefundOrderGoods.toBuilder()
                                .createUserId("0")
                                .createUserName("全量处理第三方退单商品")
                                .createTime(new Date())
                                .build();
                insertList2.add(thirdRefundOrderGoods);
            }
            if (CollectionUtil.isNotEmpty(insertList2)) {
                operationCenterOrderGoodsService.saveBatch(insertList2);
                successCount2 += insertList2.size();
            }
            offset2 += batchSize;
            insertList2 = null;
            list2 = null;
        }
        log.info("=========全量处理第三方退单商品结束, 正单总记录数：{},成功插入记录数: {}, 退单总记录数: {}， 成功插入记录数: {}=========", count, successCount, count2, successCount2);
    }


    /**
     * 根据福利商城第三方（蜂助手）订单 组装运营中心订单信息
     *
     * @param flscShopThirdOrderPay
     * @return
     */
    @Override
    public OperationCenterOrder buildOpCenterOrderFromFlscThirdOrder(FlscShopThirdOrderPay flscShopThirdOrderPay) {
        String orderSn = flscShopThirdOrderPay.getOrdersn();

        /**
         * 运营商id
         */
        Integer operatorId = flscShopThirdOrderPay.getOperatorsId();
        String operatorName = "";
        if (operatorId != null && operatorId != 0) {
            FlscShopOperatorsUser flscShopOperatorsUser = flscShopOperatorsUserMapper.selectById(operatorId);
            if (flscShopOperatorsUser != null) {
                operatorName = flscShopOperatorsUser.getOperatorsName();
            }
        }
        /**
         * 福利商城项目id
         */
        Integer organizationId = flscShopThirdOrderPay.getMemberOrganizationId();
        String organizationName = "";
        /**
         * 福利商城企业id
         */
        Integer memberEnterpriseId = flscShopThirdOrderPay.getMemberEnterpriseId();
        if (memberEnterpriseId != null && memberEnterpriseId != 0) {
            FlscShopEnterpriseUser flscShopEnterpriseUser = flscShopEnterpriseUserMapper.selectById(memberEnterpriseId);
            if (flscShopEnterpriseUser != null) {
                organizationName = StringUtils.isBlank(flscShopEnterpriseUser.getOrganName()) ? "" : flscShopEnterpriseUser.getOrganName();
            }
        }
        /**
         * 福利商城用户openid
         */
        String openid = flscShopThirdOrderPay.getOpenid();
        String buyerName = null;
        String buyerDeptName = null;
        String buyerMobile = null;
        FlscShopMember flscShopMember = flscShopMemberMapper.selectOne(Wrappers.<FlscShopMember>lambdaQuery().eq(FlscShopMember::getOpenid, openid));
        if (flscShopMember != null) {
            buyerName = flscShopMember.getRealname();
            buyerDeptName = flscShopMember.getDepartmentName();
            buyerMobile = flscShopMember.getMobile();
        }
        /**
         * 福利商城供应商id
         */
        Integer merchId = flscShopThirdOrderPay.getMerchid();
        String supplierName = "";
        FlscShopMerchUser flscShopMerchUser = flscShopMerchUserMapper.selectById(merchId);
        if (flscShopMerchUser != null) {
            supplierName = flscShopMerchUser.getMerchname();
        }

        /**
         * 平台支付方式
         */
        OperationCenterPayment platformPayment = operationCenterPaymentBizService.getPlatformPayment(OpCenterBizPlatformEnum.福利商城.getCode(), flscShopThirdOrderPay.getPaytype());


        return OperationCenterOrder.builder()
                .orderBizTypeId(OpCenterOrderBizTypeEnum.订单.getCode())//业务类型id
                .orderSn(orderSn)//订单编号
                .bizOrderId(String.valueOf(flscShopThirdOrderPay.getId()))//订单id
                .parentOrderSn(null)//父订单编号 蜂助手没有
                .parentOrderId(null)//父订单id 蜂助手没有
                .orderDataTypeId(OpCenterOrderDataTypeEnum.原始订单.getCode())//订单数据类型id
                .thirdPartyOrderSn(flscShopThirdOrderPay.getThirdOrdersn())//三方单号
                .bizPlatformCode(OpCenterBizPlatformEnum.福利商城.getCode())//业务平台编码
                .bizPlatformName(OpCenterBizPlatformEnum.福利商城.getName())//业务平台名称
                .orderTypeCode(OpCenterOrderTypeEnum.卡劵商城.getCode())//订单类型编码
                .orderTypeName(OpCenterOrderTypeEnum.卡劵商城.getName())//订单类型名称
                .bizSourceId(OpCenterBizSourceEnum.福利商城.getCode())//业务来源id
                .bizSourceName(OpCenterBizSourceEnum.福利商城.getDesc())//业务来源名称
                .bizTerminalId(OpCenterBizTerminalEnum.微信小程序.getCode())//业务终端id
                .bizTerminalCode(null)//业务终端编码，福利商城置空
                .bizTerminalName(null)//业务终端名称，福利商城置空
                .orderCreateTime(new Date(flscShopThirdOrderPay.getCreatetime() * 1000L))//订单时间
                .tradingTime(flscShopThirdOrderPay.getPaytime() > 0 ? new Date(flscShopThirdOrderPay.getPaytime() * 1000L) : null)//订单支付时间
                .sendTime(null)//发货时间 蜂助手没有
                .finishTime(flscShopThirdOrderPay.getPaytime() > 0 ? new Date(flscShopThirdOrderPay.getPaytime() * 1000L) : null)//完成时间 取支付时间
                .operatorId(String.valueOf(operatorId))//运营商id（业务方）
                .operatorName(operatorName)//运营商名称（业务方） 福利商城从ims_superdesk_shop_operators_user表里取
                .orgId(organizationId == null ? null : String.valueOf(organizationId))//项目id
                .orgName(organizationName)//项目名称
                .enterpriseId(memberEnterpriseId == null ? null : String.valueOf(memberEnterpriseId))//企业id
                .enterpriseName(flscShopThirdOrderPay.getMemberEnterpriseName())//企业名称
                .buyerId(openid)//下单人userId 福利商城取openid
                .buyerName(buyerName)//下单人姓名
                .buyerDeptId(null)//下单人部门id 福利商城没有
                .buyerDeptName(buyerDeptName)//下单人部门名称
                .buyerMobile(buyerMobile)//下单人手机号
                .supplierId(String.valueOf(merchId))//供应商id
                .supplierName(supplierName)//供应商名称
                .orderGoodsAmount(flscShopThirdOrderPay.getPrice())//订单所有商品销售总价之和，蜂助手
                .orderNegotiatedAmount(flscShopThirdOrderPay.getPrice())//订单协议金额
                .orderPrice(flscShopThirdOrderPay.getPrice())//订单金额（实付金额）
                .orderSettlementAmount(flscShopThirdOrderPay.getPrice())//订单结算金额
                .orderCostPrice(flscShopThirdOrderPay.getPrice())//订单成本金额
                .transportAmount(BigDecimal.ZERO)//运费
                .merchantTransportAmount(BigDecimal.ZERO)// 商户承担运费 只有京东订单有
                .discountAmount(BigDecimal.ZERO)//优惠金额： 商品表原价累加 + 运费 - 订单实付
                .couponId(null)//优惠券id
                .discountDesc(null)//优惠券描述
                .platformAftersaleStatusId(OpCenterPlatformAftersaleStatusEnum.未售后.getCode())//平台售后状态id
                .platformAftersaleTypeId(null)//平台售后类型id
                .platformAftersaleType(null)//平台售后类型
                .bizOrderPayTypeId(flscShopThirdOrderPay.getPaytype() == null ? null : String.valueOf(flscShopThirdOrderPay.getPaytype()))//业务订单支付方式id
                .bizOrderPayType(FlscPayTypeEnum.getNameById(flscShopThirdOrderPay.getPaytype()))//业务订单支付方式
                .platformPayTypeId(platformPayment.getId())//平台支付方式id
                .platformPayType(platformPayment.getName())//平台支付方式名称
                .orderPayStatus(FlscThirdOrderPayStatusEnum.getOpOrderPayStatusIdByBizId(flscShopThirdOrderPay.getStatus()))//订单支付状态
                .bizOrderStatusId(flscShopThirdOrderPay.getStatus() == null ? null : String.valueOf(flscShopThirdOrderPay.getStatus()))//业务订单状态id
                .bizOrderStatusName(FlscThirdOrderPayStatusEnum.getBizOrderPayStatusDescByBizId(flscShopThirdOrderPay.getStatus()))//业务订单状态名称
                .transactionStatusId(FlscThirdOrderPayStatusEnum.getOpTransactionDescByBizId(flscShopThirdOrderPay.getStatus()))//交易状态id
                .orderRemark(null)//订单备注
                .merchantRemark(null)//商家备注
                .consignee(null)//收货人
                .consigneeMobile(null)//收货人电话
                .consigneeAddress(null)
                .consigneeAddressId(null)//收货地址id
                .province(null)//省
                .city(null)//市
                .county(null)//区/县
                .street(null)//街道
                .addressDetail(null)
                .distributionTypeId(null)//配送方式id
                .distributionType(null)//配送方式名称
                .expressCode(null)//物流快递编码
                .expressName(null)//物流快递名称
                .expressTrackingNo(null)//物流快递单号
                .refundOriginOrderSn(null)//退单原单号 正单为空
                .deleted(0)
                .build();
    }


    /**
     * 根据福利商城第三方（蜂助手）订单 组装运营中心订单商品
     *
     * @param flscShopThirdOrderPay
     * @return
     */
    @Override
    public OperationCenterOrderGoods buildOpCenterOrderGoodsFromFlscThirdOrder(FlscShopThirdOrderPay flscShopThirdOrderPay) {

        /**
         * 运营商id
         */
        Integer operatorId = flscShopThirdOrderPay.getOperatorsId();
        String operatorName = "";
        if (operatorId != null && operatorId != 0) {
            FlscShopOperatorsUser flscShopOperatorsUser = flscShopOperatorsUserMapper.selectById(operatorId);
            if (flscShopOperatorsUser != null) {
                operatorName = flscShopOperatorsUser.getOperatorsName();
            }
        }

        /**
         * 福利商城供应商id
         */
        Integer merchId = flscShopThirdOrderPay.getMerchid();
        String supplierName = "";
        FlscShopMerchUser flscShopMerchUser = flscShopMerchUserMapper.selectById(merchId);
        if (flscShopMerchUser != null) {
            supplierName = flscShopMerchUser.getMerchname();
        }
        //支付金额
        BigDecimal price = flscShopThirdOrderPay.getPrice();
        return OperationCenterOrderGoods.builder()
                .bizTypeId(OpCenterOrderBizTypeEnum.订单.getCode())//业务类型id
                .orderGoodsId(OpCenterOrderTypeEnum.卡劵商城.getCode() + flscShopThirdOrderPay.getId())//订单商品id
                .orderSn(flscShopThirdOrderPay.getOrdersn())//订单编号
                .orderId(String.valueOf(flscShopThirdOrderPay.getId()))
                .parentOrderId(null)//父订单id
                .parentOrderSn(null)//父订单编号
                .thirdPartyOrderSn(flscShopThirdOrderPay.getThirdOrdersn())//第三方订单编号
                .bizPlatformName(OpCenterBizPlatformEnum.福利商城.getName())//业务平台名称
                .bizPlatformCode(OpCenterBizPlatformEnum.福利商城.getCode())//业务平台编码
                .orderTypeCode(OpCenterOrderTypeEnum.卡劵商城.getCode())//订单类型编码
                .orderTypeName(OpCenterOrderTypeEnum.卡劵商城.getName())//订单类型名称
                .orderCreateTime(new Date(flscShopThirdOrderPay.getCreatetime() * 1000L))//订单创建时间
                .finishTime(flscShopThirdOrderPay.getPaytime() > 0 ? new Date(flscShopThirdOrderPay.getPaytime() * 1000L) : null)//订单完成时间
                .isPackage(null)//是否套餐
                .packageId(null)//套餐id
                .packageName(null)//套餐名称
                .goodsTypeId(OpCenterGoodsTypeEnum.商品.getCode())//商品类型id
                .goodsId(String.valueOf(OpCenterCustomizeGoodsEnum.卡券类商品.getCode()))//商品id
                .goodsName(OpCenterCustomizeGoodsEnum.卡券类商品.getDesc())//商品名称
                .goodsSku(null)//商品sku
                .quantity(1)//数量
                .unit("项")//单位
                .originalUnitPrice(price)//原单价
                .originalTotalPrice(price)//原总价
                .sellingTotalPrice(price)//售价总价
                .sellingUnitPrice(price)//售价单价
                .negotiatedTotalPrice(price)//协议总价
                .negotiatedUnitPrice(price)//协议单价
                .settlementAmount(price)//结算金额
                .settlementUnitPrice(price)//结算单价
                .costUnitPrice(price)//成本单价
                .costTotalPrice(price)//成本总价
                .discountAmount(BigDecimal.ZERO)//优惠金额
                .platformTaxCode(null)//平台税务编码
                .platformTaxRate(BigDecimal.ZERO)//平台税率
                .merchantTaxCode(null)//商户税务编码
                .merchantTaxRate(BigDecimal.ZERO)//商户税率
                .isDutyFree(OpCenterIsDutyFreeEnum.不免税.getCode())//默认不免税
                .operatorId(String.valueOf(operatorId))//运营商id
                .operatorName(operatorName)//运营商名称
                .supplierId(String.valueOf(merchId))//供应商id
                .supplierName(supplierName)//供应商名称
                .brandId(null)//品牌id
                .brandName(null)//品牌名称
                .cateLevel1Id(null)//一级分类id
                .cateLevel1Name(null)//一级分类名称
                .cateLevel2Id(null)//二级分类id
                .cateLevel2Name(null)//二级分类名称
                .cateLevel3Id(null)//三级分类id
                .cateLevel3Name(null)//三级分类名称
                .goodsSpecification(null)//商品规格
                .otherAttribute(null)//其他属性
                .refundOriginOrderSn(null)//退款原订单编号 正单商品为空
                .build();
    }

    @Override
    public OperationCenterOrderGoods buildOpCenterRefundOrderGoodsFromFlscThirdOrder(FlscShopThirdOrderRefund flscShopThirdOrderRefund, FlscShopThirdOrderPay flscShopThirdOrderPay) {
        /**
         * 运营商id
         */
        Integer operatorId = flscShopThirdOrderPay.getOperatorsId();
        String operatorName = "";
        if (operatorId != null && operatorId != 0) {
            FlscShopOperatorsUser flscShopOperatorsUser = flscShopOperatorsUserMapper.selectById(operatorId);
            if (flscShopOperatorsUser != null) {
                operatorName = flscShopOperatorsUser.getOperatorsName();
            }
        }

        /**
         * 福利商城供应商id
         */
        Integer merchId = flscShopThirdOrderPay.getMerchid();
        String supplierName = "";
        FlscShopMerchUser flscShopMerchUser = flscShopMerchUserMapper.selectById(merchId);
        if (flscShopMerchUser != null) {
            supplierName = flscShopMerchUser.getMerchname();
        }

        return OperationCenterOrderGoods.builder()
                .bizTypeId(OpCenterOrderBizTypeEnum.售后.getCode())//业务类型id
                .orderGoodsId(OpCenterOrderTypeEnum.卡劵商城.getCode() + flscShopThirdOrderRefund.getId())//订单商品id
                .orderSn(flscShopThirdOrderRefund.getRefundno())//订单编号
                .orderId(String.valueOf(flscShopThirdOrderRefund.getId()))
                .parentOrderId(null)//父订单id
                .parentOrderSn(null)//父订单编号
                .thirdPartyOrderSn(flscShopThirdOrderRefund.getThirdRefundno())//第三方订单编号
                .bizPlatformName(OpCenterBizPlatformEnum.福利商城.getName())//业务平台名称
                .bizPlatformCode(OpCenterBizPlatformEnum.福利商城.getCode())//业务平台编码
                .orderTypeCode(OpCenterOrderTypeEnum.卡劵商城.getCode())//订单类型编码
                .orderTypeName(OpCenterOrderTypeEnum.卡劵商城.getName())//订单类型名称
                .orderCreateTime(new Date(flscShopThirdOrderPay.getCreatetime() * 1000L))//订单创建时间
                .finishTime(flscShopThirdOrderPay.getPaytime() > 0 ? new Date(flscShopThirdOrderPay.getPaytime() * 1000L) : null)//订单完成时间
                .isPackage(null)//是否套餐
                .packageId(null)//套餐id
                .packageName(null)//套餐名称
                .goodsTypeId(OpCenterGoodsTypeEnum.商品.getCode())//商品类型id
                .goodsId(String.valueOf(OpCenterCustomizeGoodsEnum.卡券类商品.getCode()))//商品id
                .goodsName(OpCenterCustomizeGoodsEnum.卡券类商品.getDesc())//商品名称
                .goodsSku(null)//商品sku
                .quantity(-1)//数量 负数
                .unit("项")//单位
                .originalUnitPrice(new BigDecimal(flscShopThirdOrderRefund.getPrice()))//原单价
                .originalTotalPrice(new BigDecimal(flscShopThirdOrderRefund.getPrice()).negate())//原总价 负数
                .sellingTotalPrice(new BigDecimal(flscShopThirdOrderRefund.getPrice()).negate())//售价总价 负数
                .sellingUnitPrice(new BigDecimal(flscShopThirdOrderRefund.getPrice()))//售价单价
                .negotiatedTotalPrice(new BigDecimal(flscShopThirdOrderRefund.getPrice()).negate())//协议总价 负数
                .negotiatedUnitPrice(new BigDecimal(flscShopThirdOrderRefund.getPrice()))//协议单价
                .settlementUnitPrice(new BigDecimal(flscShopThirdOrderRefund.getPrice()))//结算单价
                .settlementAmount(new BigDecimal(flscShopThirdOrderRefund.getPrice()).negate())//结算总价 负数
                .costTotalPrice(new BigDecimal(flscShopThirdOrderRefund.getPrice()).negate())//成本总价 负数
                .costUnitPrice(new BigDecimal(flscShopThirdOrderRefund.getPrice()))//成本单价
                .discountAmount(BigDecimal.ZERO)//优惠金额
                .platformTaxCode(null)//平台税务编码
                .platformTaxRate(BigDecimal.ZERO)//平台税率
                .merchantTaxCode(null)//商户税务编码
                .merchantTaxRate(BigDecimal.ZERO)//商户税率
                .isDutyFree(OpCenterIsDutyFreeEnum.不免税.getCode())//是否免税
                .operatorId(String.valueOf(operatorId))//运营商id
                .operatorName(operatorName)//运营商名称
                .supplierId(String.valueOf(merchId))//供应商id
                .supplierName(supplierName)//供应商名称
                .brandId(null)//品牌id
                .brandName(null)//品牌名称
                .cateLevel1Id(null)//一级分类id
                .cateLevel1Name(null)//一级分类名称
                .cateLevel2Id(null)//二级分类id
                .cateLevel2Name(null)//二级分类名称
                .cateLevel3Id(null)//三级分类id
                .cateLevel3Name(null)//三级分类名称
                .goodsSpecification(null)//商品规格
                .otherAttribute(null)//其他属性
                .refundOriginOrderSn(flscShopThirdOrderPay.getOrdersn())//退款原订单编号
                .build();
    }
}
