package cn.superdesk.data.extract.service.impl.flsc;

import cn.hutool.core.collection.CollectionUtil;
import cn.superdesk.data.extract.entity.flsc.*;
import cn.superdesk.data.extract.entity.opCenter.OperationCenterOrder;
import cn.superdesk.data.extract.entity.opCenter.OperationCenterPayment;
import cn.superdesk.data.extract.enums.flsc.FlscPayTypeEnum;
import cn.superdesk.data.extract.enums.flsc.FlscThirdAfterSaleStatusEnum;
import cn.superdesk.data.extract.enums.opCenter.*;
import cn.superdesk.data.extract.mapper.flsc.*;
import cn.superdesk.data.extract.mapper.opCenter.OperationCenterOrderMapper;
import cn.superdesk.data.extract.service.flsc.FlscShopThirdOrderRefundService;
import cn.superdesk.data.extract.service.opCenter.OperationCenterOperatorSettlementStrategyService;
import cn.superdesk.data.extract.service.opCenter.OperationCenterOrderService;
import cn.superdesk.data.extract.service.opCenter.OperationCenterPaymentBizService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author storyxc
 * @since 2023-03-17
 */
@Service
@RequiredArgsConstructor
@Slf4j
public class FlscShopThirdOrderRefundServiceImpl extends ServiceImpl<FlscShopThirdOrderRefundMapper, FlscShopThirdOrderRefund> implements FlscShopThirdOrderRefundService {
    private final FlscShopThirdOrderRefundMapper flscShopThirdOrderRefundMapper;
    private final OperationCenterOrderService operationCenterOrderService;
    private final FlscShopThirdOrderPayMapper flscShopThirdOrderPayMapper;
    private final FlscShopOperatorsUserMapper flscShopOperatorsUserMapper;
    private final FlscShopEnterpriseUserMapper flscShopEnterpriseUserMapper;
    private final FlscShopMemberMapper flscShopMemberMapper;
    private final FlscShopMerchUserMapper flscShopMerchUserMapper;
    private final FlscShopOrderGoodsMapper flscShopOrderGoodsMapper;
    private final OperationCenterOperatorSettlementStrategyService operationCenterOperatorSettlementStrategyService;
    private final OperationCenterPaymentBizService operationCenterPaymentBizService;
    private final OperationCenterOrderMapper operationCenterOrderMapper;


    /**
     * 全量处理第三方退单（蜂助手）
     *
     * @return
     */
    @Override
    @Async
    @Transactional(rollbackFor = Exception.class)
    public void handleAllRefundThirdOrder() {
        Integer count = flscShopThirdOrderRefundMapper.selectTotalCount();
        int batchSize = 1000;
        int mod = count % batchSize;
        int time = mod > 0 ? count / batchSize + 1 : count / batchSize;
        log.info("总条数 ===== {}", count);
        log.info("处理批次数 ===== {}", time);

        //成功处理记录数
        int successCount = 0;

        int offset = 0;
        List<FlscShopThirdOrderRefund> list = null;
        List<OperationCenterOrder> insertList = null;
        for (int i = 0; i < time; i++) {
            list = flscShopThirdOrderRefundMapper.selectByBatch(offset, batchSize);
            insertList = new ArrayList<>();

            for (FlscShopThirdOrderRefund flscShopThirdOrderRefund : list) {
                /**
                 * 对应正单的主键id
                 */
                Integer orderId = flscShopThirdOrderRefund.getOrderid();
                if (orderId < 0) {
                    continue;
                }
                FlscShopThirdOrderPay flscShopThirdOrderPay = flscShopThirdOrderPayMapper.selectById(orderId);
                //组装第三方退单
                OperationCenterOrder opCenterOrder = buildOpCenterRefundOrderFromFlscThirdOrder(flscShopThirdOrderRefund, flscShopThirdOrderPay);
                opCenterOrder = opCenterOrder.toBuilder()
                        .createUserId("0")
                        .createUserName("全量处理第三方退单")
                        .createTime(new Date())
                        .build();
                insertList.add(opCenterOrder);

                /**
                 * 反写正单状态
                 */
                List<FlscShopThirdOrderRefund> flscShopThirdOrderRefundList = flscShopThirdOrderRefundMapper.selectList(
                        Wrappers.<FlscShopThirdOrderRefund>lambdaQuery()
                                .eq(FlscShopThirdOrderRefund::getOrderid, flscShopThirdOrderPay.getId())
                                .eq(FlscShopThirdOrderRefund::getSourceFrom, OpCenterOrderTypeEnum.卡劵商城.getCode())
                );
                if (CollectionUtil.isNotEmpty(flscShopThirdOrderRefundList)) {
                    OperationCenterOrder operationCenterOrderExist = operationCenterOrderMapper.selectOne(Wrappers.<OperationCenterOrder>lambdaQuery()
                            .eq(OperationCenterOrder::getOrderSn, flscShopThirdOrderPay.getOrdersn())
                            .eq(OperationCenterOrder::getBizPlatformCode, OpCenterBizPlatformEnum.福利商城.getCode())
                            .eq(OperationCenterOrder::getOrderTypeCode, OpCenterOrderTypeEnum.卡劵商城.getCode())
                    );
                    if (operationCenterOrderExist != null && operationCenterOrderExist.getId() != null) {
                        List<Integer> statusList = flscShopThirdOrderRefundList.stream().map(FlscShopThirdOrderRefund::getStatus).distinct().collect(Collectors.toList());
                        if (CollectionUtil.isNotEmpty(statusList) && statusList.size() > 0) {
                            if ((statusList.size() == 1 && statusList.contains(FlscThirdAfterSaleStatusEnum.已通过.getCode())) ||
                                    (statusList.size() == 2 && statusList.contains(FlscThirdAfterSaleStatusEnum.已通过.getCode()) && statusList.contains(FlscThirdAfterSaleStatusEnum.拒绝.getCode()))) {
                                //1.全部完成 / 部分完成 部分拒绝
                                operationCenterOrderExist.setPlatformAftersaleStatusId(OpCenterPlatformAftersaleStatusEnum.售后完成.getCode());
                            } else if (statusList.size() == 1 && statusList.contains(FlscThirdAfterSaleStatusEnum.拒绝.getCode())) {
                                //2.全部拒绝
                                operationCenterOrderExist.setPlatformAftersaleStatusId(OpCenterPlatformAftersaleStatusEnum.售后取消.getCode());
                            } else {
                                //3.全部售后中
                                //4.部分售后中
                                operationCenterOrderExist.setPlatformAftersaleStatusId(OpCenterPlatformAftersaleStatusEnum.售后中.getCode());
                            }
                            //反写售后单状态
                            operationCenterOrderExist.setUpdateUserId("1");
                            operationCenterOrderExist.setUpdateUserName("同步退单反写售后状态");
                            operationCenterOrderMapper.updateById(operationCenterOrderExist);
                        }
                    }
                }
            }

            operationCenterOrderService.saveBatch(insertList);
            successCount += insertList.size();
            insertList = null;
            offset += batchSize;
            list = null;
        }
        log.info("=======================全量处理第三方（蜂助手）退单完成, 总记录数：{}, 成功处理记录数: {}=======================", count, successCount);
    }

    /**
     * 组装退单
     *
     * @param flscShopThirdOrderRefund
     * @param flscShopThirdOrderPay
     * @return
     */
    @Override
    public OperationCenterOrder buildOpCenterRefundOrderFromFlscThirdOrder(FlscShopThirdOrderRefund flscShopThirdOrderRefund, FlscShopThirdOrderPay flscShopThirdOrderPay) {


        /**
         * 订单退款时间
         */
        Date tradingTime = FlscThirdAfterSaleStatusEnum.已通过.getCode().equals(flscShopThirdOrderRefund.getStatus())
                && flscShopThirdOrderRefund.getOperatetime() > 0 ? new Date(flscShopThirdOrderRefund.getOperatetime() * 1000L) : null;

        /**
         * 福利商城运营商id
         */
        Integer operatorId = flscShopThirdOrderPay.getOperatorsId();
        String operatorName = "";
        if (operatorId != null && operatorId != 0) {
            FlscShopOperatorsUser flscShopOperatorsUser = flscShopOperatorsUserMapper.selectById(operatorId);
            if (flscShopOperatorsUser != null) {
                operatorName = flscShopOperatorsUser.getOperatorsName();
            }
        }
        /**
         * 福利商城项目id
         */
        Integer organizationId = flscShopThirdOrderPay.getMemberOrganizationId();
        String organizationName = "";
        /**
         * 福利商城企业id
         */
        Integer memberEnterpriseId = flscShopThirdOrderPay.getMemberEnterpriseId();
        if (memberEnterpriseId != null && memberEnterpriseId != 0) {
            FlscShopEnterpriseUser flscShopEnterpriseUser = flscShopEnterpriseUserMapper.selectById(memberEnterpriseId);
            if (flscShopEnterpriseUser != null) {
                organizationName = StringUtils.isBlank(flscShopEnterpriseUser.getOrganName()) ? "" : flscShopEnterpriseUser.getOrganName();
            }
        }
        /**
         * 福利商城用户openid
         */
        String openid = flscShopThirdOrderPay.getOpenid();
        String buyerName = null;
        String buyerDeptName = null;
        String buyerMobile = null;
        FlscShopMember flscShopMember = flscShopMemberMapper.selectOne(Wrappers.<FlscShopMember>lambdaQuery().eq(FlscShopMember::getOpenid, openid));
        if (flscShopMember != null) {
            buyerName = flscShopMember.getRealname();
            buyerDeptName = flscShopMember.getDepartmentName();
            buyerMobile = flscShopMember.getMobile();
        }
        /**
         * 福利商城供应商id
         */
        Integer merchId = flscShopThirdOrderPay.getMerchid();
        String supplierName = "";
        FlscShopMerchUser flscShopMerchUser = flscShopMerchUserMapper.selectById(merchId);
        if (flscShopMerchUser != null) {
            supplierName = flscShopMerchUser.getMerchname();
        }
        /**
         * 计算售后单相关金额
         */


        BigDecimal orderGoodsAmount = flscShopThirdOrderPay.getPrice();//订单商品金额
        BigDecimal orderNegotiatedAmount = flscShopThirdOrderPay.getPrice();//订单协议金额
        BigDecimal orderCostPrice = flscShopThirdOrderPay.getPrice();//订单成本金额
        BigDecimal orderPrice = new BigDecimal(flscShopThirdOrderRefund.getPrice());//订单金额（实付） 售后单取申请退款金额
        BigDecimal dispatchPrice = BigDecimal.ZERO;//售后运费
        BigDecimal merchantTransportAmount = BigDecimal.ZERO;//售后商户承担运费
        BigDecimal discountAmount = BigDecimal.ZERO;//优惠金额
        /**
         * 结算金额，第三方订单因为协议价销售价都一样 所以直接取了
         */
        BigDecimal orderSettlementAmount = flscShopThirdOrderPay.getPrice();


        /**
         * 平台支付方式
         */
        OperationCenterPayment platformPayment = operationCenterPaymentBizService.getPlatformPayment(OpCenterBizPlatformEnum.福利商城.getCode(), flscShopThirdOrderPay.getPaytype());


        return OperationCenterOrder.builder()
                .orderBizTypeId(OpCenterOrderBizTypeEnum.售后.getCode())//业务类型id
                .orderSn(flscShopThirdOrderRefund.getRefundno())//订单编号 售后单取售后单号
                .bizOrderId(String.valueOf(flscShopThirdOrderRefund.getId()))//订单id
                .parentOrderSn(null)//父订单编号 售后单为空
                .parentOrderId(null)//父订单id 售后单为空
                .orderDataTypeId(OpCenterOrderDataTypeEnum.原始订单.getCode())//订单数据类型id
                .thirdPartyOrderSn(flscShopThirdOrderRefund.getThirdRefundno())//三方单号 第三方售后编号
                .bizPlatformCode(OpCenterBizPlatformEnum.福利商城.getCode())//业务平台编码
                .bizPlatformName(OpCenterBizPlatformEnum.福利商城.getName())//业务平台名称
                .orderTypeCode(OpCenterOrderTypeEnum.卡劵商城.getCode())//订单类型编码
                .orderTypeName(OpCenterOrderTypeEnum.卡劵商城.getName())//订单类型名称
                .bizSourceId(OpCenterBizSourceEnum.卡劵商城.getCode())//业务来源id
                .bizSourceName(OpCenterBizSourceEnum.卡劵商城.getDesc())//业务来源名称
                .bizTerminalId(OpCenterBizTerminalEnum.微信小程序.getCode())//业务终端id
                .bizTerminalCode(null)//业务终端编码，福利商城置空
                .bizTerminalName(null)//业务终端名称，福利商城置空
                .orderCreateTime(flscShopThirdOrderRefund.getCreatetime() > 0 ? new Date(flscShopThirdOrderRefund.getCreatetime() * 1000L) : null)//订单时间 售后单的创建时间
                .tradingTime(tradingTime)//订单支付/退款时间
                .sendTime(null)//发货时间
                .finishTime(flscShopThirdOrderRefund.getOperatetime() > 0 ? new Date(flscShopThirdOrderRefund.getOperatetime() * 1000L) : null)//完成时间
                .operatorId(String.valueOf(operatorId))//运营商id（业务方）
                .operatorName(operatorName)//运营商名称（业务方） 福利商城从ims_superdesk_shop_operators_user表里取
                .orgId(organizationId == null ? null : String.valueOf(organizationId))//项目id
                .orgName(organizationName)//项目名称
                .enterpriseId(memberEnterpriseId == null ? null : String.valueOf(memberEnterpriseId))//企业id
                .enterpriseName(flscShopThirdOrderPay.getMemberEnterpriseName())//企业名称
                .buyerId(openid)//下单人userId 福利商城取openid
                .buyerName(buyerName)//下单人姓名
                .buyerDeptId(null)//下单人部门id 福利商城没有
                .buyerDeptName(buyerDeptName)//下单人部门名称
                .buyerMobile(buyerMobile)//下单人手机号
                .supplierId(String.valueOf(merchId))//供应商id
                .supplierName(supplierName)//供应商名称
                .orderGoodsAmount(orderGoodsAmount.negate())//订单所有商品销售总价之和，不含运费 售后单为负数
                .orderNegotiatedAmount(orderNegotiatedAmount.negate())//订单协议金额 售后单为负数
                .orderPrice(orderPrice.negate())//订单金额（实付金额）  售后单取退款金额
                .orderSettlementAmount(orderSettlementAmount.negate())//订单结算金额
                .orderCostPrice(orderCostPrice.negate())//订单成本金额
                .transportAmount(dispatchPrice)//运费
                .merchantTransportAmount(merchantTransportAmount)// 商户承担运费 只有京东订单有
                .discountAmount(discountAmount)//优惠金额
                .couponId(null)//优惠券id
                .discountDesc(null)//优惠券描述
                .platformAftersaleStatusId(FlscThirdAfterSaleStatusEnum.getOpStatusIdByBizStatusId(flscShopThirdOrderRefund.getStatus()))//平台售后状态id
                .platformAftersaleTypeId(OpCenterPlatformAftersaleTypeEnum.退货退款.getCode())//平台售后类型id
                .platformAftersaleType(OpCenterPlatformAftersaleTypeEnum.退货退款.getDesc())//平台售后类型
                .bizOrderPayTypeId(String.valueOf(flscShopThirdOrderPay.getPaytype()))//业务订单支付方式id
                .bizOrderPayType(FlscPayTypeEnum.getNameById(flscShopThirdOrderPay.getPaytype()))//业务订单支付方式
                .platformPayTypeId(platformPayment.getId())//平台支付方式id
                .platformPayType(platformPayment.getName())//平台支付方式名称
                .orderPayStatus(FlscThirdAfterSaleStatusEnum.已通过.getCode().equals(flscShopThirdOrderRefund.getStatus()) ? OpCenterOrderPayStatusEnum.已退款.getCode() : OpCenterOrderPayStatusEnum.未退款.getCode())//订单支付状态 / 退款状态
                .bizOrderStatusId(String.valueOf(flscShopThirdOrderRefund.getStatus()))//业务订单状态id
                .bizOrderStatusName(FlscThirdAfterSaleStatusEnum.getBizOrderStatusDescByBizId(flscShopThirdOrderRefund.getStatus()))//业务订单状态名称
                .transactionStatusId(FlscThirdAfterSaleStatusEnum.getOpTransactionIdByBizId(flscShopThirdOrderRefund.getStatus()))//交易状态id
                .orderRemark(flscShopThirdOrderRefund.getRemark())//订单备注
                .merchantRemark(flscShopThirdOrderRefund.getReply())//商家备注
                .consignee(null)//收货人
                .consigneeMobile(null)//收货人电话
                .consigneeAddress(null)
                .consigneeAddressId(null)//收货地址id
                .province(null)//省
                .city(null)//市
                .county(null)//区/县
                .street(null)//街道
                .addressDetail(null)
                .distributionTypeId(null)//配送方式id
                .distributionType(null)//配送方式名称
                .expressCode(null)//物流快递编码
                .expressName(null)//物流快递名称
                .expressTrackingNo(null)//物流快递单号
                .refundOriginOrderSn(flscShopThirdOrderPay.getOrdersn())//退单原单号
                .deleted(0)
                .build();
    }
}
