package cn.superdesk.data.extract.service.impl.opCenter;

import cn.hutool.core.util.ObjectUtil;
import cn.superdesk.data.extract.entity.opCenter.OperationCenterOperatorSettlementStrategy;
import cn.superdesk.data.extract.entity.opCenter.UnifiedOperators;
import cn.superdesk.data.extract.entity.opCenter.OperationCenterSettlementStrategy;
import cn.superdesk.data.extract.mapper.opCenter.OperationCenterOperatorSettlementStrategyMapper;
import cn.superdesk.data.extract.mapper.opCenter.OperationCenterOperatorsMapper;
import cn.superdesk.data.extract.mapper.opCenter.OperationCenterSettlementStrategyMapper;
import cn.superdesk.data.extract.service.opCenter.OperationCenterOperatorSettlementStrategyService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;

/**
 * <p>
 * 运营中心-运营商结算策略关系表 服务实现类
 * </p>
 *
 * @author storyxc
 * @since 2023-03-20
 */
@Service
@RequiredArgsConstructor
@Slf4j
public class OperationCenterOperatorSettlementStrategyServiceImpl extends ServiceImpl<OperationCenterOperatorSettlementStrategyMapper, OperationCenterOperatorSettlementStrategy> implements OperationCenterOperatorSettlementStrategyService {
    private final OperationCenterOperatorsMapper operationCenterOperatorsMapper;
    private final OperationCenterOperatorSettlementStrategyMapper operationCenterOperatorSettlementStrategyMapper;
    private final OperationCenterSettlementStrategyMapper operationCenterSettlementStrategyMapper;

    /**
     * 根据运营商配置的结算策略 - 获取订单结算金额
     *
     * @param operatorId            运营商id （业务方的）
     * @param bizPlatformCode       业务平台编码
     * @param orderPrice            订单实付金额
     * @param orderNegotiatedAmount 订单协议金额
     * @return
     */
    @Override
    public BigDecimal getOrderSettlementAmount(Integer operatorId, String bizPlatformCode, BigDecimal orderPrice, BigDecimal orderNegotiatedAmount) {
        UnifiedOperators operator = operationCenterOperatorsMapper.selectOne(Wrappers.<UnifiedOperators>lambdaQuery()
                .eq(UnifiedOperators::getBizPlatformCode, bizPlatformCode)
                .eq(UnifiedOperators::getBizOperatorId, operatorId));
        OperationCenterSettlementStrategy operationCenterSettlementStrategy = null;
        OperationCenterOperatorSettlementStrategy operationCenterOperatorSettlementStrategy = null;
        Integer orderSettlementAmount = 0;
        if (operator != null) {
            // 根据运营商id(运营中心的)查询是否配置了结算策略
            operationCenterOperatorSettlementStrategy = operationCenterOperatorSettlementStrategyMapper.selectOne(Wrappers.<OperationCenterOperatorSettlementStrategy>lambdaQuery().eq(OperationCenterOperatorSettlementStrategy::getOperatorId, operator.getId()));
            if (operationCenterOperatorSettlementStrategy != null) {
                operationCenterSettlementStrategy = operationCenterSettlementStrategyMapper.selectById(operationCenterOperatorSettlementStrategy.getStrategyId());
                orderSettlementAmount = operationCenterSettlementStrategy.getOrderSettlementAmount();
            }
        }
        if (operationCenterOperatorSettlementStrategy == null) {
            operationCenterSettlementStrategy = operationCenterSettlementStrategyMapper.selectOne(Wrappers.<OperationCenterSettlementStrategy>lambdaQuery()
                    .eq(OperationCenterSettlementStrategy::getIsDefault, 1)
                    .eq(OperationCenterSettlementStrategy::getDeleted, 0)
                    .eq(OperationCenterSettlementStrategy::getEnabled, 1));
            orderSettlementAmount = operationCenterSettlementStrategy.getOrderSettlementAmount();
        }

        if (ObjectUtil.equals(1, orderSettlementAmount)) {
            return orderPrice;
        } else if (ObjectUtil.equals(2, orderSettlementAmount)) {
            return orderNegotiatedAmount;
        } else {
            return orderPrice;
        }
    }

    /**
     * 根据运营商配置的结算策略 - 获取订单商品结算总价
     *
     * @param operatorId
     * @param bizPlatformCode
     * @param sellingTotalPrice
     * @param negotiatedTotalPrice
     * @return
     */
    @Override
    public BigDecimal getOrderGoodsSettlementAmount(String operatorId, String bizPlatformCode, BigDecimal sellingTotalPrice, BigDecimal negotiatedTotalPrice) {
        UnifiedOperators operator = operationCenterOperatorsMapper.selectOne(Wrappers.<UnifiedOperators>lambdaQuery()
                .eq(UnifiedOperators::getBizPlatformCode, bizPlatformCode)
                .eq(UnifiedOperators::getBizOperatorId, operatorId));
        Integer orderSettlementAmount = 0;
        OperationCenterSettlementStrategy operationCenterSettlementStrategy = null;
        OperationCenterOperatorSettlementStrategy operationCenterOperatorSettlementStrategy = null;
        if (operator != null) {
            // 根据运营商id(运营中心的)查询是否配置了结算策略
            operationCenterOperatorSettlementStrategy = operationCenterOperatorSettlementStrategyMapper.selectOne(Wrappers.<OperationCenterOperatorSettlementStrategy>lambdaQuery()
                    .eq(OperationCenterOperatorSettlementStrategy::getOperatorId, operator.getId()));
            if (operationCenterOperatorSettlementStrategy != null) {
                operationCenterSettlementStrategy = operationCenterSettlementStrategyMapper.selectById(operationCenterOperatorSettlementStrategy.getStrategyId());
                orderSettlementAmount = operationCenterSettlementStrategy.getOrderSettlementAmount();
            }
        }
        if (operationCenterSettlementStrategy == null) {
            operationCenterSettlementStrategy = operationCenterSettlementStrategyMapper.selectOne(Wrappers.<OperationCenterSettlementStrategy>lambdaQuery()
                    .eq(OperationCenterSettlementStrategy::getIsDefault, 1)
                    .eq(OperationCenterSettlementStrategy::getDeleted, 0)
                    .eq(OperationCenterSettlementStrategy::getEnabled, 1));
            orderSettlementAmount = operationCenterSettlementStrategy.getOrderSettlementAmount();
        }
        if (ObjectUtil.equals(1, orderSettlementAmount)) {
            return sellingTotalPrice;
        } else if (ObjectUtil.equals(2, orderSettlementAmount)) {
            return negotiatedTotalPrice;
        } else {
            return sellingTotalPrice;
        }
    }

    /**
     * 根据运营商配置的结算策略 - 获取订单商品结算单价
     *
     * @param operatorId          运营商id 业务方的
     * @param bizPlatformCode     业务平台编码
     * @param sellingUnitPrice    销售单价
     * @param negotiatedUnitPrice 协议单价
     * @return
     */
    @Override
    public BigDecimal getOrderGoodsSettlementUnitPrice(String operatorId, String bizPlatformCode, BigDecimal sellingUnitPrice, BigDecimal negotiatedUnitPrice) {
        UnifiedOperators operator = operationCenterOperatorsMapper.selectOne(Wrappers.<UnifiedOperators>lambdaQuery()
                .eq(UnifiedOperators::getBizPlatformCode, bizPlatformCode)
                .eq(UnifiedOperators::getBizOperatorId, operatorId));
        OperationCenterSettlementStrategy operationCenterSettlementStrategy = null;
        OperationCenterOperatorSettlementStrategy operationCenterOperatorSettlementStrategy = null;
        Integer orderSettlementAmount = 0;
        if (operator != null) {
            // 根据运营商id(运营中心的)查询是否配置了结算策略
            operationCenterOperatorSettlementStrategy = operationCenterOperatorSettlementStrategyMapper.selectOne(Wrappers.<OperationCenterOperatorSettlementStrategy>lambdaQuery()
                    .eq(OperationCenterOperatorSettlementStrategy::getOperatorId, operator.getId()));
            if (operationCenterOperatorSettlementStrategy != null) {
                operationCenterSettlementStrategy = operationCenterSettlementStrategyMapper.selectById(operationCenterOperatorSettlementStrategy.getStrategyId());
                orderSettlementAmount = operationCenterSettlementStrategy.getOrderSettlementAmount();

            }
        }
        if (operationCenterSettlementStrategy == null) {
            operationCenterSettlementStrategy = operationCenterSettlementStrategyMapper.selectOne(Wrappers.<OperationCenterSettlementStrategy>lambdaQuery()
                    .eq(OperationCenterSettlementStrategy::getIsDefault, 1)
                    .eq(OperationCenterSettlementStrategy::getDeleted, 0)
                    .eq(OperationCenterSettlementStrategy::getEnabled, 1));
            orderSettlementAmount = operationCenterSettlementStrategy.getOrderSettlementAmount();
        }
        if (ObjectUtil.equals(1, orderSettlementAmount)) {
            return sellingUnitPrice;
        } else if (ObjectUtil.equals(2, orderSettlementAmount)) {
            return negotiatedUnitPrice;
        } else {
            return sellingUnitPrice;
        }
    }
}
